/*
 * Copyright (c) 2007-2011 The Khronos Group Inc.
 * Copyright (C) 2010-2011 Linaro Ltd.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

/*
 * Authored-by: Jammy Zhou <jammy.zhou@linaro.org>
 */

#ifndef _GL_PROXY_H
#define _GL_PROXY_H

#ifdef __cplusplus
extern "C" {
#endif

/* -----------------------------------------------------------------------
 * enumeration type for backend options
 * ---------------------------------------------------------------------*/
typedef enum {
	UNKNOWN_BACKEND = -1,
	OPENGL_BACKEND,
	OPENGL_ES20_BACKEND,
	MAX_BACKEND = OPENGL_ES20_BACKEND,
} PROXY_BACKEND_OPTION;

/* ----------------------------------------------------------------------
 * proxy context
 * --------------------------------------------------------------------*/
typedef struct {
	PROXY_BACKEND_OPTION current_backend;
	void *gles2_handle;
	void *gl_handle;
	void *x_dpy;
	void *egl_handle;
	void *egl_dpy;
	void *egl_ctx;
	void *egl_surf;
} PROXY_CONTEXT;

/* -----------------------------------------------------------------------
 * definitions from <GLES2/gl2platform.h> 
 * ---------------------------------------------------------------------*/
#ifdef __GL_EXPORTS
#define GL_APICALL
#else
#define GL_APICALL extern
#endif

#define GL_APIENTRY

/* -----------------------------------------------------------------------
 * definitions from <GLES2/gl2.h>
 * ---------------------------------------------------------------------*/

/* data type definitions */
typedef void            GLvoid;
typedef char            GLchar;
typedef unsigned int    GLenum;
typedef unsigned char   GLboolean;
typedef unsigned int    GLbitfield;
typedef signed char     GLbyte;
typedef short           GLshort;
typedef int             GLint;
typedef int             GLsizei;
typedef unsigned char   GLubyte;
typedef unsigned short  GLushort;
typedef unsigned int    GLuint;
typedef float           GLfloat;
typedef float           GLclampf;
typedef int             GLfixed;
typedef int             GLclampx;

/* GL types for handling large vertex buffer objects */
typedef int             GLintptr;
typedef int             GLsizeiptr;

/* ClearBufferMask */
#define GL_DEPTH_BUFFER_BIT               0x00000100
#define GL_STENCIL_BUFFER_BIT             0x00000400
#define GL_COLOR_BUFFER_BIT               0x00004000

/* Boolean */
#define GL_FALSE                          0
#define GL_TRUE                           1

/* BeginMode */
#define GL_POINTS                         0x0000
#define GL_LINES                          0x0001
#define GL_LINE_LOOP                      0x0002
#define GL_LINE_STRIP                     0x0003
#define GL_TRIANGLES                      0x0004
#define GL_TRIANGLE_STRIP                 0x0005
#define GL_TRIANGLE_FAN                   0x0006

/* BlendingFactorDest */
#define GL_ZERO                           0
#define GL_ONE                            1
#define GL_SRC_COLOR                      0x0300
#define GL_ONE_MINUS_SRC_COLOR            0x0301
#define GL_SRC_ALPHA                      0x0302
#define GL_ONE_MINUS_SRC_ALPHA            0x0303
#define GL_DST_ALPHA                      0x0304
#define GL_ONE_MINUS_DST_ALPHA            0x0305

/* BlendingFactorSrc */
#define GL_DST_COLOR                      0x0306
#define GL_ONE_MINUS_DST_COLOR            0x0307
#define GL_SRC_ALPHA_SATURATE             0x0308

/* BlendEquationSeparate */
#define GL_FUNC_ADD                       0x8006
#define GL_BLEND_EQUATION                 0x8009
#define GL_BLEND_EQUATION_RGB             0x8009    /* same as BLEND_EQUATION */
#define GL_BLEND_EQUATION_ALPHA           0x883D

/* BlendSubtract */
#define GL_FUNC_SUBTRACT                  0x800A
#define GL_FUNC_REVERSE_SUBTRACT          0x800B

/* Separate Blend Functions */
#define GL_BLEND_DST_RGB                  0x80C8
#define GL_BLEND_SRC_RGB                  0x80C9
#define GL_BLEND_DST_ALPHA                0x80CA
#define GL_BLEND_SRC_ALPHA                0x80CB
#define GL_CONSTANT_COLOR                 0x8001
#define GL_ONE_MINUS_CONSTANT_COLOR       0x8002
#define GL_CONSTANT_ALPHA                 0x8003
#define GL_ONE_MINUS_CONSTANT_ALPHA       0x8004
#define GL_BLEND_COLOR                    0x8005

/* Buffer Objects */
#define GL_ARRAY_BUFFER                   0x8892
#define GL_ELEMENT_ARRAY_BUFFER           0x8893
#define GL_ARRAY_BUFFER_BINDING           0x8894
#define GL_ELEMENT_ARRAY_BUFFER_BINDING   0x8895

#define GL_STREAM_DRAW                    0x88E0
#define GL_STATIC_DRAW                    0x88E4
#define GL_DYNAMIC_DRAW                   0x88E8

#define GL_BUFFER_SIZE                    0x8764
#define GL_BUFFER_USAGE                   0x8765

#define GL_CURRENT_VERTEX_ATTRIB          0x8626

/* CullFaceMode */
#define GL_FRONT                          0x0404
#define GL_BACK                           0x0405
#define GL_FRONT_AND_BACK                 0x0408

/* EnableCap */
#define GL_TEXTURE_2D                     0x0DE1
#define GL_CULL_FACE                      0x0B44
#define GL_BLEND                          0x0BE2
#define GL_DITHER                         0x0BD0
#define GL_STENCIL_TEST                   0x0B90
#define GL_DEPTH_TEST                     0x0B71
#define GL_SCISSOR_TEST                   0x0C11
#define GL_POLYGON_OFFSET_FILL            0x8037
#define GL_SAMPLE_ALPHA_TO_COVERAGE       0x809E
#define GL_SAMPLE_COVERAGE                0x80A0

/* ErrorCode */
#define GL_NO_ERROR                       0
#define GL_INVALID_ENUM                   0x0500
#define GL_INVALID_VALUE                  0x0501
#define GL_INVALID_OPERATION              0x0502
#define GL_OUT_OF_MEMORY                  0x0505

/* FrontFaceDirection */
#define GL_CW                             0x0900
#define GL_CCW                            0x0901

/* GetPName */
#define GL_LINE_WIDTH                     0x0B21
#define GL_ALIASED_POINT_SIZE_RANGE       0x846D
#define GL_ALIASED_LINE_WIDTH_RANGE       0x846E
#define GL_CULL_FACE_MODE                 0x0B45
#define GL_FRONT_FACE                     0x0B46
#define GL_DEPTH_RANGE                    0x0B70
#define GL_DEPTH_WRITEMASK                0x0B72
#define GL_DEPTH_CLEAR_VALUE              0x0B73
#define GL_DEPTH_FUNC                     0x0B74
#define GL_STENCIL_CLEAR_VALUE            0x0B91
#define GL_STENCIL_FUNC                   0x0B92
#define GL_STENCIL_FAIL                   0x0B94
#define GL_STENCIL_PASS_DEPTH_FAIL        0x0B95
#define GL_STENCIL_PASS_DEPTH_PASS        0x0B96
#define GL_STENCIL_REF                    0x0B97
#define GL_STENCIL_VALUE_MASK             0x0B93
#define GL_STENCIL_WRITEMASK              0x0B98
#define GL_STENCIL_BACK_FUNC              0x8800
#define GL_STENCIL_BACK_FAIL              0x8801
#define GL_STENCIL_BACK_PASS_DEPTH_FAIL   0x8802
#define GL_STENCIL_BACK_PASS_DEPTH_PASS   0x8803
#define GL_STENCIL_BACK_REF               0x8CA3
#define GL_STENCIL_BACK_VALUE_MASK        0x8CA4
#define GL_STENCIL_BACK_WRITEMASK         0x8CA5
#define GL_VIEWPORT                       0x0BA2
#define GL_SCISSOR_BOX                    0x0C10

/* GL_SCISSOR_TEST */
#define GL_COLOR_CLEAR_VALUE              0x0C22
#define GL_COLOR_WRITEMASK                0x0C23
#define GL_UNPACK_ALIGNMENT               0x0CF5
#define GL_PACK_ALIGNMENT                 0x0D05
#define GL_MAX_TEXTURE_SIZE               0x0D33
#define GL_MAX_VIEWPORT_DIMS              0x0D3A
#define GL_SUBPIXEL_BITS                  0x0D50
#define GL_RED_BITS                       0x0D52
#define GL_GREEN_BITS                     0x0D53
#define GL_BLUE_BITS                      0x0D54
#define GL_ALPHA_BITS                     0x0D55
#define GL_DEPTH_BITS                     0x0D56
#define GL_STENCIL_BITS                   0x0D57
#define GL_POLYGON_OFFSET_UNITS           0x2A00

/* GL_POLYGON_OFFSET_FILL */
#define GL_POLYGON_OFFSET_FACTOR          0x8038
#define GL_TEXTURE_BINDING_2D             0x8069
#define GL_SAMPLE_BUFFERS                 0x80A8
#define GL_SAMPLES                        0x80A9
#define GL_SAMPLE_COVERAGE_VALUE          0x80AA
#define GL_SAMPLE_COVERAGE_INVERT         0x80AB

#define GL_NUM_COMPRESSED_TEXTURE_FORMATS 0x86A2
#define GL_COMPRESSED_TEXTURE_FORMATS     0x86A3

/* HintMode */
#define GL_DONT_CARE                      0x1100
#define GL_FASTEST                        0x1101
#define GL_NICEST                         0x1102

/* HintTarget */
#define GL_GENERATE_MIPMAP_HINT           0x8192

/* DataType */
#define GL_BYTE                           0x1400
#define GL_UNSIGNED_BYTE                  0x1401
#define GL_SHORT                          0x1402
#define GL_UNSIGNED_SHORT                 0x1403
#define GL_INT                            0x1404
#define GL_UNSIGNED_INT                   0x1405
#define GL_FLOAT                          0x1406
#define GL_FIXED                          0x140C

/* PixelFormat */
#define GL_DEPTH_COMPONENT                0x1902
#define GL_ALPHA                          0x1906
#define GL_RGB                            0x1907
#define GL_RGBA                           0x1908
#define GL_LUMINANCE                      0x1909
#define GL_LUMINANCE_ALPHA                0x190A

/* PixelType */
#define GL_UNSIGNED_SHORT_4_4_4_4         0x8033
#define GL_UNSIGNED_SHORT_5_5_5_1         0x8034
#define GL_UNSIGNED_SHORT_5_6_5           0x8363

/* Shaders */
#define GL_FRAGMENT_SHADER                  0x8B30
#define GL_VERTEX_SHADER                    0x8B31
#define GL_MAX_VERTEX_ATTRIBS               0x8869
#define GL_MAX_VERTEX_UNIFORM_VECTORS       0x8DFB
#define GL_MAX_VARYING_VECTORS              0x8DFC
#define GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS 0x8B4D
#define GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS   0x8B4C
#define GL_MAX_TEXTURE_IMAGE_UNITS          0x8872
#define GL_MAX_FRAGMENT_UNIFORM_VECTORS     0x8DFD
#define GL_SHADER_TYPE                      0x8B4F
#define GL_DELETE_STATUS                    0x8B80
#define GL_LINK_STATUS                      0x8B82
#define GL_VALIDATE_STATUS                  0x8B83
#define GL_ATTACHED_SHADERS                 0x8B85
#define GL_ACTIVE_UNIFORMS                  0x8B86
#define GL_ACTIVE_UNIFORM_MAX_LENGTH        0x8B87
#define GL_ACTIVE_ATTRIBUTES                0x8B89
#define GL_ACTIVE_ATTRIBUTE_MAX_LENGTH      0x8B8A
#define GL_SHADING_LANGUAGE_VERSION         0x8B8C
#define GL_CURRENT_PROGRAM                  0x8B8D

/* StencilFunction */
#define GL_NEVER                          0x0200
#define GL_LESS                           0x0201
#define GL_EQUAL                          0x0202
#define GL_LEQUAL                         0x0203
#define GL_GREATER                        0x0204
#define GL_NOTEQUAL                       0x0205
#define GL_GEQUAL                         0x0206
#define GL_ALWAYS                         0x0207

/* StencilOp */
#define GL_KEEP                           0x1E00
#define GL_REPLACE                        0x1E01
#define GL_INCR                           0x1E02
#define GL_DECR                           0x1E03
#define GL_INVERT                         0x150A
#define GL_INCR_WRAP                      0x8507
#define GL_DECR_WRAP                      0x8508

/* StringName */
#define GL_VENDOR                         0x1F00
#define GL_RENDERER                       0x1F01
#define GL_VERSION                        0x1F02
#define GL_EXTENSIONS                     0x1F03

/* TextureMagFilter */
#define GL_NEAREST                        0x2600
#define GL_LINEAR                         0x2601

/* TextureMinFilter */
#define GL_NEAREST_MIPMAP_NEAREST         0x2700
#define GL_LINEAR_MIPMAP_NEAREST          0x2701
#define GL_NEAREST_MIPMAP_LINEAR          0x2702
#define GL_LINEAR_MIPMAP_LINEAR           0x2703

/* TextureParameterName */
#define GL_TEXTURE_MAG_FILTER             0x2800
#define GL_TEXTURE_MIN_FILTER             0x2801
#define GL_TEXTURE_WRAP_S                 0x2802
#define GL_TEXTURE_WRAP_T                 0x2803

/* TextureTarget */
/*      GL_TEXTURE_2D */
#define GL_TEXTURE                        0x1702

#define GL_TEXTURE_CUBE_MAP               0x8513
#define GL_TEXTURE_BINDING_CUBE_MAP       0x8514
#define GL_TEXTURE_CUBE_MAP_POSITIVE_X    0x8515
#define GL_TEXTURE_CUBE_MAP_NEGATIVE_X    0x8516
#define GL_TEXTURE_CUBE_MAP_POSITIVE_Y    0x8517
#define GL_TEXTURE_CUBE_MAP_NEGATIVE_Y    0x8518
#define GL_TEXTURE_CUBE_MAP_POSITIVE_Z    0x8519
#define GL_TEXTURE_CUBE_MAP_NEGATIVE_Z    0x851A
#define GL_MAX_CUBE_MAP_TEXTURE_SIZE      0x851C

/* TextureUnit */
#define GL_TEXTURE0                       0x84C0
#define GL_TEXTURE1                       0x84C1
#define GL_TEXTURE2                       0x84C2
#define GL_TEXTURE3                       0x84C3
#define GL_TEXTURE4                       0x84C4
#define GL_TEXTURE5                       0x84C5
#define GL_TEXTURE6                       0x84C6
#define GL_TEXTURE7                       0x84C7
#define GL_TEXTURE8                       0x84C8
#define GL_TEXTURE9                       0x84C9
#define GL_TEXTURE10                      0x84CA
#define GL_TEXTURE11                      0x84CB
#define GL_TEXTURE12                      0x84CC
#define GL_TEXTURE13                      0x84CD
#define GL_TEXTURE14                      0x84CE
#define GL_TEXTURE15                      0x84CF
#define GL_TEXTURE16                      0x84D0
#define GL_TEXTURE17                      0x84D1
#define GL_TEXTURE18                      0x84D2
#define GL_TEXTURE19                      0x84D3
#define GL_TEXTURE20                      0x84D4
#define GL_TEXTURE21                      0x84D5
#define GL_TEXTURE22                      0x84D6
#define GL_TEXTURE23                      0x84D7
#define GL_TEXTURE24                      0x84D8
#define GL_TEXTURE25                      0x84D9
#define GL_TEXTURE26                      0x84DA
#define GL_TEXTURE27                      0x84DB
#define GL_TEXTURE28                      0x84DC
#define GL_TEXTURE29                      0x84DD
#define GL_TEXTURE30                      0x84DE
#define GL_TEXTURE31                      0x84DF
#define GL_ACTIVE_TEXTURE                 0x84E0

/* TextureWrapMode */
#define GL_REPEAT                         0x2901
#define GL_CLAMP_TO_EDGE                  0x812F
#define GL_MIRRORED_REPEAT                0x8370

/* Uniform Types */
#define GL_FLOAT_VEC2                     0x8B50
#define GL_FLOAT_VEC3                     0x8B51
#define GL_FLOAT_VEC4                     0x8B52
#define GL_INT_VEC2                       0x8B53
#define GL_INT_VEC3                       0x8B54
#define GL_INT_VEC4                       0x8B55
#define GL_BOOL                           0x8B56
#define GL_BOOL_VEC2                      0x8B57
#define GL_BOOL_VEC3                      0x8B58
#define GL_BOOL_VEC4                      0x8B59
#define GL_FLOAT_MAT2                     0x8B5A
#define GL_FLOAT_MAT3                     0x8B5B
#define GL_FLOAT_MAT4                     0x8B5C
#define GL_SAMPLER_2D                     0x8B5E
#define GL_SAMPLER_CUBE                   0x8B60

/* Vertex Arrays */
#define GL_VERTEX_ATTRIB_ARRAY_ENABLED        0x8622
#define GL_VERTEX_ATTRIB_ARRAY_SIZE           0x8623
#define GL_VERTEX_ATTRIB_ARRAY_STRIDE         0x8624
#define GL_VERTEX_ATTRIB_ARRAY_TYPE           0x8625
#define GL_VERTEX_ATTRIB_ARRAY_NORMALIZED     0x886A
#define GL_VERTEX_ATTRIB_ARRAY_POINTER        0x8645
#define GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING 0x889F

/* Read Format */
#define GL_IMPLEMENTATION_COLOR_READ_TYPE   0x8B9A
#define GL_IMPLEMENTATION_COLOR_READ_FORMAT 0x8B9B

/* Shader Source */
#define GL_COMPILE_STATUS                 0x8B81
#define GL_INFO_LOG_LENGTH                0x8B84
#define GL_SHADER_SOURCE_LENGTH           0x8B88
#define GL_SHADER_COMPILER                0x8DFA

/* Shader Binary */
#define GL_PLATFORM_BINARY                0x8D63
#define GL_SHADER_BINARY_FORMATS          0x8DF8
#define GL_NUM_SHADER_BINARY_FORMATS      0x8DF9

/* Shader Precision-Specified Types */
#define GL_LOW_FLOAT                      0x8DF0
#define GL_MEDIUM_FLOAT                   0x8DF1
#define GL_HIGH_FLOAT                     0x8DF2
#define GL_LOW_INT                        0x8DF3
#define GL_MEDIUM_INT                     0x8DF4
#define GL_HIGH_INT                       0x8DF5

/* Framebuffer Object. */
#define GL_FRAMEBUFFER                    0x8D40
#define GL_RENDERBUFFER                   0x8D41

#define GL_RGBA4                          0x8056
#define GL_RGB5_A1                        0x8057
#define GL_RGB565                         0x8D62
#define GL_DEPTH_COMPONENT16              0x81A5
#define GL_STENCIL_INDEX                  0x1901
#define GL_STENCIL_INDEX8                 0x8D48

#define GL_RENDERBUFFER_WIDTH             0x8D42
#define GL_RENDERBUFFER_HEIGHT            0x8D43
#define GL_RENDERBUFFER_INTERNAL_FORMAT   0x8D44
#define GL_RENDERBUFFER_RED_SIZE          0x8D50
#define GL_RENDERBUFFER_GREEN_SIZE        0x8D51
#define GL_RENDERBUFFER_BLUE_SIZE         0x8D52
#define GL_RENDERBUFFER_ALPHA_SIZE        0x8D53
#define GL_RENDERBUFFER_DEPTH_SIZE        0x8D54
#define GL_RENDERBUFFER_STENCIL_SIZE      0x8D55

#define GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE           0x8CD0
#define GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME           0x8CD1
#define GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL         0x8CD2
#define GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE 0x8CD3

#define GL_COLOR_ATTACHMENT0              0x8CE0
#define GL_DEPTH_ATTACHMENT               0x8D00
#define GL_STENCIL_ATTACHMENT             0x8D20

#define GL_FRAMEBUFFER_COMPLETE                      0x8CD5
#define GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT         0x8CD6
#define GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT 0x8CD7
#define GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS         0x8CD9
#define GL_FRAMEBUFFER_INCOMPLETE_FORMATS            0x8CDA
#define GL_FRAMEBUFFER_UNSUPPORTED                   0x8CDD

#define GL_FRAMEBUFFER_BINDING            0x8CA6
#define GL_RENDERBUFFER_BINDING           0x8CA7
#define GL_MAX_RENDERBUFFER_SIZE          0x84E8

#define GL_INVALID_FRAMEBUFFER_OPERATION  0x0506

/* ----------------------------------------------------------------------
 * funtion type definitions
 * --------------------------------------------------------------------*/

typedef void         GL_APIENTRY (*PFNPROXYACTIVETEXTUREPROC) (GLenum texture);
typedef void         GL_APIENTRY (*PFNPROXYATTACHSHADERPROC) (GLuint program, GLuint shader);
typedef void         GL_APIENTRY (*PFNPROXYBINDATTRIBLOCATIONPROC) (GLuint program, GLuint index, const char* name);
typedef void         GL_APIENTRY (*PFNPROXYBINDBUFFERPROC) (GLenum target, GLuint buffer);
typedef void         GL_APIENTRY (*PFNPROXYBINDFRAMEBUFFERPROC) (GLenum target, GLuint framebuffer);
typedef void         GL_APIENTRY (*PFNPROXYBINDRENDERBUFFERPROC) (GLenum target, GLuint renderbuffer);
typedef void         GL_APIENTRY (*PFNPROXYBINDTEXTUREPROC) (GLenum target, GLuint texture);
typedef void         GL_APIENTRY (*PFNPROXYBLENDCOLORPROC) (GLclampf red, GLclampf green, GLclampf blue, GLclampf alpha);
typedef void         GL_APIENTRY (*PFNPROXYBLENDEQUATIONPROC) ( GLenum mode );
typedef void         GL_APIENTRY (*PFNPROXYBLENDEQUATIONSEPARATEPROC) (GLenum modeRGB, GLenum modeAlpha);
typedef void         GL_APIENTRY (*PFNPROXYBLENDFUNCPROC) (GLenum sfactor, GLenum dfactor);
typedef void         GL_APIENTRY (*PFNPROXYBLENDFUNCSEPARATEPROC) (GLenum srcRGB, GLenum dstRGB, GLenum srcAlpha, GLenum dstAlpha);
typedef void         GL_APIENTRY (*PFNPROXYBUFFERDATAPROC) (GLenum target, GLsizeiptr size, const void* data, GLenum usage);
typedef void         GL_APIENTRY (*PFNPROXYBUFFERSUBDATAPROC) (GLenum target, GLintptr offset, GLsizeiptr size, const void* data);
typedef GLenum       GL_APIENTRY (*PFNPROXYCHECKFRAMEBUFFERSTATUSPROC) (GLenum target);
typedef void         GL_APIENTRY (*PFNPROXYCLEARPROC) (GLbitfield mask);
typedef void         GL_APIENTRY (*PFNPROXYCLEARCOLORPROC) (GLclampf red, GLclampf green, GLclampf blue, GLclampf alpha);
typedef void         GL_APIENTRY (*PFNPROXYCLEARDEPTHFPROC) (GLclampf depth);
typedef void         GL_APIENTRY (*PFNPROXYCLEARSTENCILPROC) (GLint s);
typedef void         GL_APIENTRY (*PFNPROXYCOLORMASKPROC) (GLboolean red, GLboolean green, GLboolean blue, GLboolean alpha);
typedef void         GL_APIENTRY (*PFNPROXYCOMPILESHADERPROC) (GLuint shader);
typedef void         GL_APIENTRY (*PFNPROXYCOMPRESSEDTEXIMAGE2DPROC) (GLenum target, GLint level, GLenum internalformat, GLsizei width, GLsizei height, GLint border, GLsizei imageSize, const void* data);
typedef void         GL_APIENTRY (*PFNPROXYCOMPRESSEDTEXSUBIMAGE2DPROC) (GLenum target, GLint level, GLint xoffset, GLint yoffset, GLsizei width, GLsizei height, GLenum format, GLsizei imageSize, const void* data);
typedef void         GL_APIENTRY (*PFNPROXYCOPYTEXIMAGE2DPROC) (GLenum target, GLint level, GLenum internalformat, GLint x, GLint y, GLsizei width, GLsizei height, GLint border);
typedef void         GL_APIENTRY (*PFNPROXYCOPYTEXSUBIMAGE2DPROC) (GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint x, GLint y, GLsizei width, GLsizei height);
typedef GLuint       GL_APIENTRY (*PFNPROXYCREATEPROGRAMPROC) (void);
typedef GLuint       GL_APIENTRY (*PFNPROXYCREATESHADERPROC) (GLenum type);
typedef void         GL_APIENTRY (*PFNPROXYCULLFACEPROC) (GLenum mode);
typedef void         GL_APIENTRY (*PFNPROXYDELETEBUFFERSPROC) (GLsizei n, const GLuint* buffers);
typedef void         GL_APIENTRY (*PFNPROXYDELETEFRAMEBUFFERSPROC) (GLsizei n, const GLuint* framebuffers);
typedef void         GL_APIENTRY (*PFNPROXYDELETEPROGRAMPROC) (GLuint program);
typedef void         GL_APIENTRY (*PFNPROXYDELETERENDERBUFFERSPROC) (GLsizei n, const GLuint* renderbuffers);
typedef void         GL_APIENTRY (*PFNPROXYDELETESHADERPROC) (GLuint shader);
typedef void         GL_APIENTRY (*PFNPROXYDELETETEXTURESPROC) (GLsizei n, const GLuint* textures);
typedef void         GL_APIENTRY (*PFNPROXYDEPTHFUNCPROC) (GLenum func);
typedef void         GL_APIENTRY (*PFNPROXYDEPTHMASKPROC) (GLboolean flag);
typedef void         GL_APIENTRY (*PFNPROXYDEPTHRANGEFPROC) (GLclampf zNear, GLclampf zFar);
typedef void         GL_APIENTRY (*PFNPROXYDETACHSHADERPROC) (GLuint program, GLuint shader);
typedef void         GL_APIENTRY (*PFNPROXYDISABLEPROC) (GLenum cap);
typedef void         GL_APIENTRY (*PFNPROXYDISABLEVERTEXATTRIBARRAYPROC) (GLuint index);
typedef void         GL_APIENTRY (*PFNPROXYDRAWARRAYSPROC) (GLenum mode, GLint first, GLsizei count);
typedef void         GL_APIENTRY (*PFNPROXYDRAWELEMENTSPROC) (GLenum mode, GLsizei count, GLenum type, const void* indices);
typedef void         GL_APIENTRY (*PFNPROXYENABLEPROC) (GLenum cap);
typedef void         GL_APIENTRY (*PFNPROXYENABLEVERTEXATTRIBARRAYPROC) (GLuint index);
typedef void         GL_APIENTRY (*PFNPROXYFINISHPROC) (void);
typedef void         GL_APIENTRY (*PFNPROXYFLUSHPROC) (void);
typedef void         GL_APIENTRY (*PFNPROXYFRAMEBUFFERRENDERBUFFERPROC) (GLenum target, GLenum attachment, GLenum renderbuffertarget, GLuint renderbuffer);
typedef void         GL_APIENTRY (*PFNPROXYFRAMEBUFFERTEXTURE2DPROC) (GLenum target, GLenum attachment, GLenum textarget, GLuint texture, GLint level);
typedef void         GL_APIENTRY (*PFNPROXYFRONTFACEPROC) (GLenum mode);
typedef void         GL_APIENTRY (*PFNPROXYGENBUFFERSPROC) (GLsizei n, GLuint* buffers);
typedef void         GL_APIENTRY (*PFNPROXYGENERATEMIPMAPPROC) (GLenum target);
typedef void         GL_APIENTRY (*PFNPROXYGENFRAMEBUFFERSPROC) (GLsizei n, GLuint* framebuffers);
typedef void         GL_APIENTRY (*PFNPROXYGENRENDERBUFFERSPROC) (GLsizei n, GLuint* renderbuffers);
typedef void         GL_APIENTRY (*PFNPROXYGENTEXTURESPROC) (GLsizei n, GLuint* textures);
typedef void         GL_APIENTRY (*PFNPROXYGETACTIVEATTRIBPROC) (GLuint program, GLuint index, GLsizei bufsize, GLsizei* length, GLint* size, GLenum* type, char* name);
typedef void         GL_APIENTRY (*PFNPROXYGETACTIVEUNIFORMPROC) (GLuint program, GLuint index, GLsizei bufsize, GLsizei* length, GLint* size, GLenum* type, char* name);
typedef void         GL_APIENTRY (*PFNPROXYGETATTACHEDSHADERSPROC) (GLuint program, GLsizei maxcount, GLsizei* count, GLuint* shaders);
typedef int          GL_APIENTRY (*PFNPROXYGETATTRIBLOCATIONPROC) (GLuint program, const char* name);
typedef void         GL_APIENTRY (*PFNPROXYGETBOOLEANVPROC) (GLenum pname, GLboolean* params);
typedef void         GL_APIENTRY (*PFNPROXYGETBUFFERPARAMETERIVPROC) (GLenum target, GLenum pname, GLint* params);
typedef GLenum       GL_APIENTRY (*PFNPROXYGETERRORPROC) (void);
typedef void         GL_APIENTRY (*PFNPROXYGETFLOATVPROC) (GLenum pname, GLfloat* params);
typedef void         GL_APIENTRY (*PFNPROXYGETFRAMEBUFFERATTACHMENTPARAMETERIVPROC) (GLenum target, GLenum attachment, GLenum pname, GLint* params);
typedef void         GL_APIENTRY (*PFNPROXYGETINTEGERVPROC) (GLenum pname, GLint* params);
typedef void         GL_APIENTRY (*PFNPROXYGETPROGRAMIVPROC) (GLuint program, GLenum pname, GLint* params);
typedef void         GL_APIENTRY (*PFNPROXYGETPROGRAMINFOLOGPROC) (GLuint program, GLsizei bufsize, GLsizei* length, char* infolog);
typedef void         GL_APIENTRY (*PFNPROXYGETRENDERBUFFERPARAMETERIVPROC) (GLenum target, GLenum pname, GLint* params);
typedef void         GL_APIENTRY (*PFNPROXYGETSHADERIVPROC) (GLuint shader, GLenum pname, GLint* params);
typedef void         GL_APIENTRY (*PFNPROXYGETSHADERINFOLOGPROC) (GLuint shader, GLsizei bufsize, GLsizei* length, char* infolog);
typedef void         GL_APIENTRY (*PFNPROXYGETSHADERPRECISIONFORMATPROC) (GLenum shadertype, GLenum precisiontype, GLint* range, GLint* precision);
typedef void         GL_APIENTRY (*PFNPROXYGETSHADERSOURCEPROC) (GLuint shader, GLsizei bufsize, GLsizei* length, char* source);
typedef const GLubyte* GL_APIENTRY (*PFNPROXYGETSTRINGPROC) (GLenum name);
typedef void         GL_APIENTRY (*PFNPROXYGETTEXPARAMETERFVPROC) (GLenum target, GLenum pname, GLfloat* params);
typedef void         GL_APIENTRY (*PFNPROXYGETTEXPARAMETERIVPROC) (GLenum target, GLenum pname, GLint* params);
typedef void         GL_APIENTRY (*PFNPROXYGETUNIFORMFVPROC) (GLuint program, GLint location, GLfloat* params);
typedef void         GL_APIENTRY (*PFNPROXYGETUNIFORMIVPROC) (GLuint program, GLint location, GLint* params);
typedef int          GL_APIENTRY (*PFNPROXYGETUNIFORMLOCATIONPROC) (GLuint program, const char* name);
typedef void         GL_APIENTRY (*PFNPROXYGETVERTEXATTRIBFVPROC) (GLuint index, GLenum pname, GLfloat* params);
typedef void         GL_APIENTRY (*PFNPROXYGETVERTEXATTRIBIVPROC) (GLuint index, GLenum pname, GLint* params);
typedef void         GL_APIENTRY (*PFNPROXYGETVERTEXATTRIBPOINTERVPROC) (GLuint index, GLenum pname, void** pointer);
typedef void         GL_APIENTRY (*PFNPROXYHINTPROC) (GLenum target, GLenum mode);
typedef GLboolean    GL_APIENTRY (*PFNPROXYISBUFFERPROC) (GLuint buffer);
typedef GLboolean    GL_APIENTRY (*PFNPROXYISENABLEDPROC) (GLenum cap);
typedef GLboolean    GL_APIENTRY (*PFNPROXYISFRAMEBUFFERPROC) (GLuint framebuffer);
typedef GLboolean    GL_APIENTRY (*PFNPROXYISPROGRAMPROC) (GLuint program);
typedef GLboolean    GL_APIENTRY (*PFNPROXYISRENDERBUFFERPROC) (GLuint renderbuffer);
typedef GLboolean    GL_APIENTRY (*PFNPROXYISSHADERPROC) (GLuint shader);
typedef GLboolean    GL_APIENTRY (*PFNPROXYISTEXTUREPROC) (GLuint texture);
typedef void         GL_APIENTRY (*PFNPROXYLINEWIDTHPROC) (GLfloat width);
typedef void         GL_APIENTRY (*PFNPROXYLINKPROGRAMPROC) (GLuint program);
typedef void         GL_APIENTRY (*PFNPROXYPIXELSTOREIPROC) (GLenum pname, GLint param);
typedef void         GL_APIENTRY (*PFNPROXYPOLYGONOFFSETPROC) (GLfloat factor, GLfloat units);
typedef void         GL_APIENTRY (*PFNPROXYREADPIXELSPROC) (GLint x, GLint y, GLsizei width, GLsizei height, GLenum format, GLenum type, void* pixels);
typedef void         GL_APIENTRY (*PFNPROXYRELEASESHADERCOMPILERPROC) (void);
typedef void         GL_APIENTRY (*PFNPROXYRENDERBUFFERSTORAGEPROC) (GLenum target, GLenum internalformat, GLsizei width, GLsizei height);
typedef void         GL_APIENTRY (*PFNPROXYSAMPLECOVERAGEPROC) (GLclampf value, GLboolean invert);
typedef void         GL_APIENTRY (*PFNPROXYSCISSORPROC) (GLint x, GLint y, GLsizei width, GLsizei height);
typedef void         GL_APIENTRY (*PFNPROXYSHADERBINARYPROC) (GLint n, const GLuint* shaders, GLenum binaryformat, const void* binary, GLint length);
typedef void         GL_APIENTRY (*PFNPROXYSHADERSOURCEPROC) (GLuint shader, GLsizei count, const char** string, const GLint* length);
typedef void         GL_APIENTRY (*PFNPROXYSTENCILFUNCPROC) (GLenum func, GLint ref, GLuint mask);
typedef void         GL_APIENTRY (*PFNPROXYSTENCILFUNCSEPARATEPROC) (GLenum face, GLenum func, GLint ref, GLuint mask);
typedef void         GL_APIENTRY (*PFNPROXYSTENCILMASKPROC) (GLuint mask);
typedef void         GL_APIENTRY (*PFNPROXYSTENCILMASKSEPARATEPROC) (GLenum face, GLuint mask);
typedef void         GL_APIENTRY (*PFNPROXYSTENCILOPPROC) (GLenum fail, GLenum zfail, GLenum zpass);
typedef void         GL_APIENTRY (*PFNPROXYSTENCILOPSEPARATEPROC) (GLenum face, GLenum fail, GLenum zfail, GLenum zpass);
typedef void         GL_APIENTRY (*PFNPROXYTEXIMAGE2DPROC) (GLenum target, GLint level, GLenum internalformat, GLsizei width, GLsizei height, GLint border, GLenum format, GLenum type, const void* pixels);
typedef void         GL_APIENTRY (*PFNPROXYTEXPARAMETERFPROC) (GLenum target, GLenum pname, GLfloat param);
typedef void         GL_APIENTRY (*PFNPROXYTEXPARAMETERFVPROC) (GLenum target, GLenum pname, const GLfloat* params);
typedef void         GL_APIENTRY (*PFNPROXYTEXPARAMETERIPROC) (GLenum target, GLenum pname, GLint param);
typedef void         GL_APIENTRY (*PFNPROXYTEXPARAMETERIVPROC) (GLenum target, GLenum pname, const GLint* params);
typedef void         GL_APIENTRY (*PFNPROXYTEXSUBIMAGE2DPROC) (GLenum target, GLint level, GLint xoffset, GLint yoffset, GLsizei width, GLsizei height, GLenum format, GLenum type, const void* pixels);
typedef void         GL_APIENTRY (*PFNPROXYUNIFORM1FPROC) (GLint location, GLfloat x);
typedef void         GL_APIENTRY (*PFNPROXYUNIFORM1FVPROC) (GLint location, GLsizei count, const GLfloat* v);
typedef void         GL_APIENTRY (*PFNPROXYUNIFORM1IPROC) (GLint location, GLint x);
typedef void         GL_APIENTRY (*PFNPROXYUNIFORM1IVPROC) (GLint location, GLsizei count, const GLint* v);
typedef void         GL_APIENTRY (*PFNPROXYUNIFORM2FPROC) (GLint location, GLfloat x, GLfloat y);
typedef void         GL_APIENTRY (*PFNPROXYUNIFORM2FVPROC) (GLint location, GLsizei count, const GLfloat* v);
typedef void         GL_APIENTRY (*PFNPROXYUNIFORM2IPROC) (GLint location, GLint x, GLint y);
typedef void         GL_APIENTRY (*PFNPROXYUNIFORM2IVPROC) (GLint location, GLsizei count, const GLint* v);
typedef void         GL_APIENTRY (*PFNPROXYUNIFORM3FPROC) (GLint location, GLfloat x, GLfloat y, GLfloat z);
typedef void         GL_APIENTRY (*PFNPROXYUNIFORM3FVPROC) (GLint location, GLsizei count, const GLfloat* v);
typedef void         GL_APIENTRY (*PFNPROXYUNIFORM3IPROC) (GLint location, GLint x, GLint y, GLint z);
typedef void         GL_APIENTRY (*PFNPROXYUNIFORM3IVPROC) (GLint location, GLsizei count, const GLint* v);
typedef void         GL_APIENTRY (*PFNPROXYUNIFORM4FPROC) (GLint location, GLfloat x, GLfloat y, GLfloat z, GLfloat w);
typedef void         GL_APIENTRY (*PFNPROXYUNIFORM4FVPROC) (GLint location, GLsizei count, const GLfloat* v);
typedef void         GL_APIENTRY (*PFNPROXYUNIFORM4IPROC) (GLint location, GLint x, GLint y, GLint z, GLint w);
typedef void         GL_APIENTRY (*PFNPROXYUNIFORM4IVPROC) (GLint location, GLsizei count, const GLint* v);
typedef void         GL_APIENTRY (*PFNPROXYUNIFORMMATRIX2FVPROC) (GLint location, GLsizei count, GLboolean transpose, const GLfloat* value);
typedef void         GL_APIENTRY (*PFNPROXYUNIFORMMATRIX3FVPROC) (GLint location, GLsizei count, GLboolean transpose, const GLfloat* value);
typedef void         GL_APIENTRY (*PFNPROXYUNIFORMMATRIX4FVPROC) (GLint location, GLsizei count, GLboolean transpose, const GLfloat* value);
typedef void         GL_APIENTRY (*PFNPROXYUSEPROGRAMPROC) (GLuint program);
typedef void         GL_APIENTRY (*PFNPROXYVALIDATEPROGRAMPROC) (GLuint program);
typedef void         GL_APIENTRY (*PFNPROXYVERTEXATTRIB1FPROC) (GLuint indx, GLfloat x);
typedef void         GL_APIENTRY (*PFNPROXYVERTEXATTRIB1FVPROC) (GLuint indx, const GLfloat* values);
typedef void         GL_APIENTRY (*PFNPROXYVERTEXATTRIB2FPROC) (GLuint indx, GLfloat x, GLfloat y);
typedef void         GL_APIENTRY (*PFNPROXYVERTEXATTRIB2FVPROC) (GLuint indx, const GLfloat* values);
typedef void         GL_APIENTRY (*PFNPROXYVERTEXATTRIB3FPROC) (GLuint indx, GLfloat x, GLfloat y, GLfloat z);
typedef void         GL_APIENTRY (*PFNPROXYVERTEXATTRIB3FVPROC) (GLuint indx, const GLfloat* values);
typedef void         GL_APIENTRY (*PFNPROXYVERTEXATTRIB4FPROC) (GLuint indx, GLfloat x, GLfloat y, GLfloat z, GLfloat w);
typedef void         GL_APIENTRY (*PFNPROXYVERTEXATTRIB4FVPROC) (GLuint indx, const GLfloat* values);
typedef void         GL_APIENTRY (*PFNPROXYVERTEXATTRIBPOINTERPROC) (GLuint indx, GLint size, GLenum type, GLboolean normalized, GLsizei stride, const void* ptr);
typedef void         GL_APIENTRY (*PFNPROXYVIEWPORTPROC) (GLint x, GLint y, GLsizei width, GLsizei height);

/* ----------------------------------------------------------------------
 * function rename
 * --------------------------------------------------------------------*/
#define glActiveTexture            pfnProxyActiveTexture
#define glAttachShader             pfnProxyAttachShader
#define glBindAttribLocation       pfnProxyBindAttribLocation
#define glBindBuffer               pfnProxyBindBuffer
#define glBindFramebuffer          pfnProxyBindFramebuffer
#define glBindRenderbuffer         pfnProxyBindRenderbuffer
#define glBindTexture              pfnProxyBindTexture
#define glBlendColor               pfnProxyBlendColor
#define glBlendEquation            pfnProxyBlendEquation
#define glBlendEquationSeparate    pfnProxyBlendEquationSeparate
#define glBlendFunc                pfnProxyBlendFunc
#define glBlendFuncSeparate        pfnProxyBlendFuncSeparate
#define glBufferData               pfnProxyBufferData
#define glBufferSubData            pfnProxyBufferSubData
#define glCheckFramebufferStatus   pfnProxyCheckFramebufferStatus
#define glClear                    pfnProxyClear
#define glClearColor               pfnProxyClearColor
#define glClearDepthf              pfnProxyClearDepthf
#define glClearStencil             pfnProxyClearStencil
#define glColorMask                pfnProxyColorMask
#define glCompileShader            pfnProxyCompileShader
#define glCompressedTexImage2D     pfnProxyCompressedTexImage2D
#define glCompressedTexSubImage2D  pfnProxyCompressedTexSubImage2D
#define glCopyTexImage2D           pfnProxyCopyTexImage2D
#define glCopyTexSubImage2D        pfnProxyCopyTexSubImage2D
#define glCreateProgram            pfnProxyCreateProgram
#define glCreateShader             pfnProxyCreateShader
#define glCullFace                 pfnProxyCullFace
#define glDeleteBuffers            pfnProxyDeleteBuffers
#define glDeleteFramebuffers       pfnProxyDeleteFramebuffers
#define glDeleteProgram            pfnProxyDeleteProgram
#define glDeleteRenderbuffers      pfnProxyDeleteRenderbuffers
#define glDeleteShader             pfnProxyDeleteShader
#define glDeleteTextures           pfnProxyDeleteTextures
#define glDepthFunc                pfnProxyDepthFunc
#define glDepthMask                pfnProxyDepthMask
#define glDepthRangef              pfnProxyDepthRangef
#define glDetachShader             pfnProxyDetachShader
#define glDisable                  pfnProxyDisable
#define glDisableVertexAttribArray pfnProxyDisableVertexAttribArray
#define glDrawArrays               pfnProxyDrawArrays
#define glDrawElements             pfnProxyDrawElements
#define glEnable                   pfnProxyEnable
#define glEnableVertexAttribArray  pfnProxyEnableVertexAttribArray
#define glFinish                   pfnProxyFinish
#define glFlush                    pfnProxyFlush
#define glFramebufferRenderbuffer  pfnProxyFramebufferRenderbuffer
#define glFramebufferTexture2D     pfnProxyFramebufferTexture2D
#define glFrontFace                pfnProxyFrontFace
#define glGenBuffers               pfnProxyGenBuffers
#define glGenerateMipmap           pfnProxyGenerateMipmap
#define glGenFramebuffers          pfnProxyGenFramebuffers
#define glGenRenderbuffers         pfnProxyGenRenderbuffers
#define glGenTextures              pfnProxyGenTextures
#define glGetActiveAttrib          pfnProxyGetActiveAttrib
#define glGetActiveUniform         pfnProxyGetActiveUniform
#define glGetAttachedShaders       pfnProxyGetAttachedShaders
#define glGetAttribLocation        pfnProxyGetAttribLocation
#define glGetBooleanv              pfnProxyGetBooleanv
#define glGetBufferParameteriv     pfnProxyGetBufferParameteriv
#define glGetError                 pfnProxyGetError
#define glGetFloatv                pfnProxyGetFloatv
#define glGetFramebufferAttachmentParameteriv pfnProxyGetFramebufferAttachmentParameteriv
#define glGetIntegerv              pfnProxyGetIntegerv
#define glGetProgramiv             pfnProxyGetProgramiv
#define glGetProgramInfoLog        pfnProxyGetProgramInfoLog
#define glGetRenderbufferParameteriv pfnProxyGetRenderbufferParameteriv
#define glGetShaderiv              pfnProxyGetShaderiv
#define glGetShaderInfoLog         pfnProxyGetShaderInfoLog
#define glGetShaderPrecisionFormat pfnProxyGetShaderPrecisionFormat
#define glGetShaderSource          pfnProxyGetShaderSource
#define glGetString                pfnProxyGetString
#define glGetTexParameterfv        pfnProxyGetTexParameterfv
#define glGetTexParameteriv        pfnProxyGetTexParameteriv
#define glGetUniformfv             pfnProxyGetUniformfv
#define glGetUniformiv             pfnProxyGetUniformiv
#define glGetUniformLocation       pfnProxyGetUniformLocation
#define glGetVertexAttribfv        pfnProxyGetVertexAttribfv
#define glGetVertexAttribiv        pfnProxyGetVertexAttribiv
#define glGetVertexAttribPointerv  pfnProxyGetVertexAttribPointerv
#define glHint                     pfnProxyHint
#define glIsBuffer                 pfnProxyIsBuffer
#define glIsEnabled                pfnProxyIsEnabled
#define glIsFramebuffer            pfnProxyIsFramebuffer
#define glIsProgram                pfnProxyIsProgram
#define glIsRenderbuffer           pfnProxyIsRenderbuffer
#define glIsShader                 pfnProxyIsShader
#define glIsTexture                pfnProxyIsTexture
#define glLineWidth                pfnProxyLineWidth
#define glLinkProgram              pfnProxyLinkProgram
#define glPixelStorei              pfnProxyPixelStorei
#define glPolygonOffset            pfnProxyPolygonOffset
#define glReadPixels               pfnProxyReadPixels
#define glReleaseShaderCompiler    pfnProxyReleaseShaderCompiler
#define glRenderbufferStorage      pfnProxyRenderbufferStorage
#define glSampleCoverage           pfnProxySampleCoverage
#define glScissor                  pfnProxyScissor
#define glShaderBinary             pfnProxyShaderBinary
#define glShaderSource             pfnProxyShaderSource
#define glStencilFunc              pfnProxyStencilFunc
#define glStencilFuncSeparate      pfnProxyStencilFuncSeparate
#define glStencilMask              pfnProxyStencilMask
#define glStencilMaskSeparate      pfnProxyStencilMaskSeparate
#define glStencilOp                pfnProxyStencilOp
#define glStencilOpSeparate        pfnProxyStencilOpSeparate
#define glTexImage2D               pfnProxyTexImage2D
#define glTexParameterf            pfnProxyTexParameterf
#define glTexParameterfv           pfnProxyTexParameterfv
#define glTexParameteri            pfnProxyTexParameteri
#define glTexParameteriv           pfnProxyTexParameteriv
#define glTexSubImage2D            pfnProxyTexSubImage2D
#define glUniform1f                pfnProxyUniform1f
#define glUniform1fv               pfnProxyUniform1fv
#define glUniform1i                pfnProxyUniform1i
#define glUniform1iv               pfnProxyUniform1iv
#define glUniform2f                pfnProxyUniform2f
#define glUniform2fv               pfnProxyUniform2fv
#define glUniform2i                pfnProxyUniform2i
#define glUniform2iv               pfnProxyUniform2iv
#define glUniform3f                pfnProxyUniform3f
#define glUniform3fv               pfnProxyUniform3fv
#define glUniform3i                pfnProxyUniform3i
#define glUniform3iv               pfnProxyUniform3iv
#define glUniform4f                pfnProxyUniform4f
#define glUniform4fv               pfnProxyUniform4fv
#define glUniform4i                pfnProxyUniform4i
#define glUniform4iv               pfnProxyUniform4iv
#define glUniformMatrix2fv         pfnProxyUniformMatrix2fv
#define glUniformMatrix3fv         pfnProxyUniformMatrix3fv
#define glUniformMatrix4fv         pfnProxyUniformMatrix4fv
#define glUseProgram               pfnProxyUseProgram
#define glValidateProgram          pfnProxyValidateProgram
#define glVertexAttrib1f           pfnProxyVertexAttrib1f
#define glVertexAttrib1fv          pfnProxyVertexAttrib1fv
#define glVertexAttrib2f           pfnProxyVertexAttrib2f
#define glVertexAttrib2fv          pfnProxyVertexAttrib2fv
#define glVertexAttrib3f           pfnProxyVertexAttrib3f
#define glVertexAttrib3fv          pfnProxyVertexAttrib3fv
#define glVertexAttrib4f           pfnProxyVertexAttrib4f
#define glVertexAttrib4fv          pfnProxyVertexAttrib4fv
#define glVertexAttribPointer      pfnProxyVertexAttribPointer
#define glViewport                 pfnProxyViewport

/* ----------------------------------------------------------------------
 * export proxy functions
 * --------------------------------------------------------------------*/
extern PFNPROXYACTIVETEXTUREPROC      pfnProxyActiveTexture;
extern PFNPROXYATTACHSHADERPROC       pfnProxyAttachShader;
extern PFNPROXYBINDATTRIBLOCATIONPROC pfnProxyBindAttribLocation;
extern PFNPROXYBINDBUFFERPROC         pfnProxyBindBuffer;
extern PFNPROXYBINDFRAMEBUFFERPROC    pfnProxyBindFramebuffer;
extern PFNPROXYBINDRENDERBUFFERPROC   pfnProxyBindRenderbuffer;
extern PFNPROXYBINDTEXTUREPROC        pfnProxyBindTexture;
extern PFNPROXYBLENDCOLORPROC         pfnProxyBlendColor;
extern PFNPROXYBLENDEQUATIONPROC      pfnProxyBlendEquation;
extern PFNPROXYBLENDEQUATIONSEPARATEPROC pfnProxyBlendEquationSeparate;
extern PFNPROXYBLENDFUNCPROC          pfnProxyBlendFunc;
extern PFNPROXYBLENDFUNCSEPARATEPROC  pfnProxyBlendFuncSeparate;
extern PFNPROXYBUFFERDATAPROC         pfnProxyBufferData;
extern PFNPROXYBUFFERSUBDATAPROC      pfnProxyBufferSubData;
extern PFNPROXYCHECKFRAMEBUFFERSTATUSPROC pfnProxyCheckFramebufferStatus;
extern PFNPROXYCLEARPROC              pfnProxyClear;
extern PFNPROXYCLEARCOLORPROC         pfnProxyClearColor;
extern PFNPROXYCLEARDEPTHFPROC        pfnProxyClearDepthf;
extern PFNPROXYCLEARSTENCILPROC       pfnProxyClearStencil;
extern PFNPROXYCOLORMASKPROC          pfnProxyColorMask;
extern PFNPROXYCOMPILESHADERPROC      pfnProxyCompileShader;
extern PFNPROXYCOMPRESSEDTEXIMAGE2DPROC pfnProxyCompressedTexImage2D;
extern PFNPROXYCOMPRESSEDTEXSUBIMAGE2DPROC pfnProxyCompressedTexSubImage2D;
extern PFNPROXYCOPYTEXIMAGE2DPROC     pfnProxyCopyTexImage2D;
extern PFNPROXYCOPYTEXSUBIMAGE2DPROC  pfnProxyCopyTexSubImage2D;
extern PFNPROXYCREATEPROGRAMPROC      pfnProxyCreateProgram;
extern PFNPROXYCREATESHADERPROC       pfnProxyCreateShader;
extern PFNPROXYCULLFACEPROC           pfnProxyCullFace;
extern PFNPROXYDELETEBUFFERSPROC      pfnProxyDeleteBuffers;
extern PFNPROXYDELETEFRAMEBUFFERSPROC pfnProxyDeleteFramebuffers;
extern PFNPROXYDELETEPROGRAMPROC      pfnProxyDeleteProgram;
extern PFNPROXYDELETERENDERBUFFERSPROC pfnProxyDeleteRenderbuffers;
extern PFNPROXYDELETESHADERPROC       pfnProxyDeleteShader;
extern PFNPROXYDELETETEXTURESPROC     pfnProxyDeleteTextures;
extern PFNPROXYDEPTHFUNCPROC          pfnProxyDepthFunc;
extern PFNPROXYDEPTHMASKPROC          pfnProxyDepthMask;
extern PFNPROXYDEPTHRANGEFPROC        pfnProxyDepthRangef;
extern PFNPROXYDETACHSHADERPROC       pfnProxyDetachShader;
extern PFNPROXYDISABLEPROC            pfnProxyDisable;
extern PFNPROXYDISABLEVERTEXATTRIBARRAYPROC pfnProxyDisableVertexAttribArray;
extern PFNPROXYDRAWARRAYSPROC         pfnProxyDrawArrays;
extern PFNPROXYDRAWELEMENTSPROC       pfnProxyDrawElements;
extern PFNPROXYENABLEPROC             pfnProxyEnable;
extern PFNPROXYENABLEVERTEXATTRIBARRAYPROC pfnProxyEnableVertexAttribArray;
extern PFNPROXYFINISHPROC             pfnProxyFinish;
extern PFNPROXYFLUSHPROC              pfnProxyFlush;
extern PFNPROXYFRAMEBUFFERRENDERBUFFERPROC pfnProxyFramebufferRenderbuffer;
extern PFNPROXYFRAMEBUFFERTEXTURE2DPROC pfnProxyFramebufferTexture2D;
extern PFNPROXYFRONTFACEPROC          pfnProxyFrontFace;
extern PFNPROXYGENBUFFERSPROC         pfnProxyGenBuffers;
extern PFNPROXYGENERATEMIPMAPPROC     pfnProxyGenerateMipmap;
extern PFNPROXYGENFRAMEBUFFERSPROC    pfnProxyGenFramebuffers;
extern PFNPROXYGENRENDERBUFFERSPROC   pfnProxyGenRenderbuffers;
extern PFNPROXYGENTEXTURESPROC        pfnProxyGenTextures;
extern PFNPROXYGETACTIVEATTRIBPROC    pfnProxyGetActiveAttrib;
extern PFNPROXYGETACTIVEUNIFORMPROC   pfnProxyGetActiveUniform;
extern PFNPROXYGETATTACHEDSHADERSPROC pfnProxyGetAttachedShaders;
extern PFNPROXYGETATTRIBLOCATIONPROC  pfnProxyGetAttribLocation;
extern PFNPROXYGETBOOLEANVPROC        pfnProxyGetBooleanv;
extern PFNPROXYGETBUFFERPARAMETERIVPROC pfnProxyGetBufferParameteriv;
extern PFNPROXYGETERRORPROC           pfnProxyGetError;
extern PFNPROXYGETFLOATVPROC          pfnProxyGetFloatv;
extern PFNPROXYGETFRAMEBUFFERATTACHMENTPARAMETERIVPROC pfnProxyGetFramebufferAttachmentParameteriv;
extern PFNPROXYGETINTEGERVPROC        pfnProxyGetIntegerv;
extern PFNPROXYGETPROGRAMIVPROC       pfnProxyGetProgramiv;
extern PFNPROXYGETPROGRAMINFOLOGPROC  pfnProxyGetProgramInfoLog;
extern PFNPROXYGETRENDERBUFFERPARAMETERIVPROC pfnProxyGetRenderbufferParameteriv;
extern PFNPROXYGETSHADERIVPROC        pfnProxyGetShaderiv;
extern PFNPROXYGETSHADERINFOLOGPROC  pfnProxyGetShaderInfoLog;
extern PFNPROXYGETSHADERPRECISIONFORMATPROC pfnProxyGetShaderPrecisionFormat;
extern PFNPROXYGETSHADERSOURCEPROC    pfnProxyGetShaderSource;
extern PFNPROXYGETSTRINGPROC          pfnProxyGetString;
extern PFNPROXYGETTEXPARAMETERFVPROC  pfnProxyGetTexParameterfv;
extern PFNPROXYGETTEXPARAMETERIVPROC  pfnProxyGetTexParameteriv;
extern PFNPROXYGETUNIFORMFVPROC       pfnProxyGetUniformfv;
extern PFNPROXYGETUNIFORMIVPROC       pfnProxyGetUniformiv;
extern PFNPROXYGETUNIFORMLOCATIONPROC pfnProxyGetUniformLocation;
extern PFNPROXYGETVERTEXATTRIBFVPROC  pfnProxyGetVertexAttribfv;
extern PFNPROXYGETVERTEXATTRIBIVPROC  pfnProxyGetVertexAttribiv;
extern PFNPROXYGETVERTEXATTRIBPOINTERVPROCpfnProxyGetVertexAttribPointerv;
extern PFNPROXYHINTPROC               pfnProxyHint;
extern PFNPROXYISBUFFERPROC           pfnProxyIsBuffer;
extern PFNPROXYISENABLEDPROC          pfnProxyIsEnabled;
extern PFNPROXYISFRAMEBUFFERPROC      pfnProxyIsFramebuffer;
extern PFNPROXYISPROGRAMPROC          pfnProxyIsProgram;
extern PFNPROXYISRENDERBUFFERPROC     pfnProxyIsRenderbuffer;
extern PFNPROXYISSHADERPROC           pfnProxyIsShader;
extern PFNPROXYISTEXTUREPROC          pfnProxyIsTexture;
extern PFNPROXYLINEWIDTHPROC          pfnProxyLineWidth;
extern PFNPROXYLINKPROGRAMPROC        pfnProxyLinkProgram;
extern PFNPROXYPIXELSTOREIPROC        pfnProxyPixelStorei;
extern PFNPROXYPOLYGONOFFSETPROC      pfnProxyPolygonOffset;
extern PFNPROXYREADPIXELSPROC         pfnProxyReadPixels;
extern PFNPROXYRELEASESHADERCOMPILERPROC pfnProxyReleaseShaderCompiler;
extern PFNPROXYRENDERBUFFERSTORAGEPROC pfnProxyRenderbufferStorage;
extern PFNPROXYSAMPLECOVERAGEPROC     pfnProxySampleCoverage;
extern PFNPROXYSCISSORPROC            pfnProxyScissor;
extern PFNPROXYSHADERBINARYPROC       pfnProxyShaderBinary;
extern PFNPROXYSHADERSOURCEPROC       pfnProxyShaderSource;
extern PFNPROXYSTENCILFUNCPROC        pfnProxyStencilFunc;
extern PFNPROXYSTENCILFUNCSEPARATEPROC pfnProxyStencilFuncSeparate;
extern PFNPROXYSTENCILMASKPROC        pfnProxyStencilMask;
extern PFNPROXYSTENCILMASKSEPARATEPROC pfnProxyStencilMaskSeparate;
extern PFNPROXYSTENCILOPPROC          pfnProxyStencilOp;
extern PFNPROXYSTENCILOPSEPARATEPROC  pfnProxyStencilOpSeparate;
extern PFNPROXYTEXIMAGE2DPROC         pfnProxyTexImage2D;
extern PFNPROXYTEXPARAMETERFPROC      pfnProxyTexParameterf;
extern PFNPROXYTEXPARAMETERFVPROC     pfnProxyTexParameterfv;
extern PFNPROXYTEXPARAMETERIPROC      pfnProxyTexParameteri;
extern PFNPROXYTEXPARAMETERIVPROC     pfnProxyTexParameteriv;
extern PFNPROXYTEXSUBIMAGE2DPROC      pfnProxyTexSubImage2D;
extern PFNPROXYUNIFORM1FPROC          pfnProxyUniform1f;
extern PFNPROXYUNIFORM1FVPROC         pfnProxyUniform1fv;
extern PFNPROXYUNIFORM1IPROC          pfnProxyUniform1i;
extern PFNPROXYUNIFORM1IVPROC         pfnProxyUniform1iv;
extern PFNPROXYUNIFORM2FPROC          pfnProxyUniform2f;
extern PFNPROXYUNIFORM2FVPROC         pfnProxyUniform2fv;
extern PFNPROXYUNIFORM2IPROC          pfnProxyUniform2i;
extern PFNPROXYUNIFORM2IVPROC         pfnProxyUniform2iv;
extern PFNPROXYUNIFORM3FPROC          pfnProxyUniform3f;
extern PFNPROXYUNIFORM3FVPROC         pfnProxyUniform3fv;
extern PFNPROXYUNIFORM3IPROC          pfnProxyUniform3i;
extern PFNPROXYUNIFORM3IVPROC         pfnProxyUniform3iv;
extern PFNPROXYUNIFORM4FPROC          pfnProxyUniform4f;
extern PFNPROXYUNIFORM4FVPROC         pfnProxyUniform4fv;
extern PFNPROXYUNIFORM4IPROC          pfnProxyUniform4i;
extern PFNPROXYUNIFORM4IVPROC         pfnProxyUniform4iv;
extern PFNPROXYUNIFORMMATRIX2FVPROC   pfnProxyUniformMatrix2fv;
extern PFNPROXYUNIFORMMATRIX3FVPROC   pfnProxyUniformMatrix3fv;
extern PFNPROXYUNIFORMMATRIX4FVPROC   pfnProxyUniformMatrix4fv;
extern PFNPROXYUSEPROGRAMPROC         pfnProxyUseProgram;
extern PFNPROXYVALIDATEPROGRAMPROC    pfnProxyValidateProgram;
extern PFNPROXYVERTEXATTRIB1FPROC     pfnProxyVertexAttrib1f;
extern PFNPROXYVERTEXATTRIB1FVPROC    pfnProxyVertexAttrib1fv;
extern PFNPROXYVERTEXATTRIB2FPROC     pfnProxyVertexAttrib2f;
extern PFNPROXYVERTEXATTRIB2FVPROC    pfnProxyVertexAttrib2fv;
extern PFNPROXYVERTEXATTRIB3FPROC     pfnProxyVertexAttrib3f;
extern PFNPROXYVERTEXATTRIB3FVPROC    pfnProxyVertexAttrib3fv;
extern PFNPROXYVERTEXATTRIB4FPROC     pfnProxyVertexAttrib4f;
extern PFNPROXYVERTEXATTRIB4FVPROC    pfnProxyVertexAttrib4fv;
extern PFNPROXYVERTEXATTRIBPOINTERPROC pfnProxyVertexAttribPointer;
extern PFNPROXYVIEWPORTPROC           pfnProxyViewport;

/* ---------------------------------------------------------------
 * Init and Exit functions
 * -------------------------------------------------------------*/
extern int glProxyInit(PROXY_BACKEND_OPTION *pProxyBackend);
extern int glProxyClose(void);

#ifdef __cplusplus
}
#endif

#endif
