/*
 * Copyright (c) 2007-2011 The Khronos Group Inc.
 * Copyright (C) 2010-2011 Linaro Ltd.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

/*
 * Authored-by: Jammy Zhou <jammy.zhou@linaro.org>
 */

#ifndef _GLX_PROXY_H
#define _GLX_PROXY_H

#ifdef __cplusplus
extern "C" {
#endif

#include <X11/Xlib.h>
#include <X11/Xutil.h>

/* --------------------------------------------------------------------
 * Definitions from /usr/include/GL/glx.h
 * ------------------------------------------------------------------*/

/*
 * Tokens for glXChooseVisual and glXGetConfig
 */
#define GLX_USE_GL		1
#define GLX_BUFFER_SIZE		2
#define GLX_LEVEL		3
#define GLX_RGBA		4
#define GLX_DOUBLEBUFFER	5
#define GLX_STEREO		6
#define GLX_AUX_BUFFERS		7
#define GLX_RED_SIZE		8
#define GLX_GREEN_SIZE		9
#define GLX_BLUE_SIZE		10
#define GLX_ALPHA_SIZE		11
#define GLX_DEPTH_SIZE		12
#define GLX_STENCIL_SIZE	13
#define GLX_ACCUM_RED_SIZE	14
#define GLX_ACCUM_GREEN_SIZE	15
#define GLX_ACCUM_BLUE_SIZE	16
#define GLX_ACCUM_ALPHA_SIZE	17

/*
 * Error codes returned by glXGetConfig:
 */
#define GLX_BAD_SCREEN		1
#define GLX_BAD_ATTRIBUTE	2
#define GLX_NO_EXTENSION	3
#define GLX_BAD_VISUAL		4
#define GLX_BAD_CONTEXT		5
#define GLX_BAD_VALUE       	6
#define GLX_BAD_ENUM		7

/*
 * GLX 1.1 and later:
 */
#define GLX_VENDOR		1
#define GLX_VERSION		2
#define GLX_EXTENSIONS 		3

/*
 * GLX 1.3 and later:
 */
#define GLX_CONFIG_CAVEAT		0x20
#define GLX_DONT_CARE			0xFFFFFFFF
#define GLX_X_VISUAL_TYPE		0x22
#define GLX_TRANSPARENT_TYPE		0x23
#define GLX_TRANSPARENT_INDEX_VALUE	0x24
#define GLX_TRANSPARENT_RED_VALUE	0x25
#define GLX_TRANSPARENT_GREEN_VALUE	0x26
#define GLX_TRANSPARENT_BLUE_VALUE	0x27
#define GLX_TRANSPARENT_ALPHA_VALUE	0x28
#define GLX_WINDOW_BIT			0x00000001
#define GLX_PIXMAP_BIT			0x00000002
#define GLX_PBUFFER_BIT			0x00000004
#define GLX_AUX_BUFFERS_BIT		0x00000010
#define GLX_FRONT_LEFT_BUFFER_BIT	0x00000001
#define GLX_FRONT_RIGHT_BUFFER_BIT	0x00000002
#define GLX_BACK_LEFT_BUFFER_BIT	0x00000004
#define GLX_BACK_RIGHT_BUFFER_BIT	0x00000008
#define GLX_DEPTH_BUFFER_BIT		0x00000020
#define GLX_STENCIL_BUFFER_BIT		0x00000040
#define GLX_ACCUM_BUFFER_BIT		0x00000080
#define GLX_NONE			0x8000
#define GLX_SLOW_CONFIG			0x8001
#define GLX_TRUE_COLOR			0x8002
#define GLX_DIRECT_COLOR		0x8003
#define GLX_PSEUDO_COLOR		0x8004
#define GLX_STATIC_COLOR		0x8005
#define GLX_GRAY_SCALE			0x8006
#define GLX_STATIC_GRAY			0x8007
#define GLX_TRANSPARENT_RGB		0x8008
#define GLX_TRANSPARENT_INDEX		0x8009
#define GLX_VISUAL_ID			0x800B
#define GLX_SCREEN			0x800C
#define GLX_NON_CONFORMANT_CONFIG	0x800D
#define GLX_DRAWABLE_TYPE		0x8010
#define GLX_RENDER_TYPE			0x8011
#define GLX_X_RENDERABLE		0x8012
#define GLX_FBCONFIG_ID			0x8013
#define GLX_RGBA_TYPE			0x8014
#define GLX_COLOR_INDEX_TYPE		0x8015
#define GLX_MAX_PBUFFER_WIDTH		0x8016
#define GLX_MAX_PBUFFER_HEIGHT		0x8017
#define GLX_MAX_PBUFFER_PIXELS		0x8018
#define GLX_PRESERVED_CONTENTS		0x801B
#define GLX_LARGEST_PBUFFER		0x801C
#define GLX_WIDTH			0x801D
#define GLX_HEIGHT			0x801E
#define GLX_EVENT_MASK			0x801F
#define GLX_DAMAGED			0x8020
#define GLX_SAVED			0x8021
#define GLX_WINDOW			0x8022
#define GLX_PBUFFER			0x8023
#define GLX_PBUFFER_HEIGHT              0x8040
#define GLX_PBUFFER_WIDTH               0x8041
#define GLX_RGBA_BIT			0x00000001
#define GLX_COLOR_INDEX_BIT		0x00000002
#define GLX_PBUFFER_CLOBBER_MASK	0x08000000

/*
 * GLX 1.4 and later:
 */
#define GLX_SAMPLE_BUFFERS              0x186a0 /*100000*/
#define GLX_SAMPLES                     0x186a1 /*100001*/

typedef struct __GLXcontextRec *GLXContext;
typedef XID GLXPixmap;
typedef XID GLXDrawable;

/* GLX 1.3 and later */
typedef struct __GLXFBConfigRec *GLXFBConfig;
typedef XID GLXFBConfigID;
typedef XID GLXContextID;
typedef XID GLXWindow;
typedef XID GLXPbuffer;

/*
** Events.
** __GLX_NUMBER_EVENTS is set to 17 to account for the BufferClobberSGIX
**  event - this helps initialization if the server supports the pbuffer
**  extension and the client doesn't.
*/
#define GLX_PbufferClobber	0
#define GLX_BufferSwapComplete	1

#define __GLX_NUMBER_EVENTS 17

/* ---------------------------------------------------------------------
 * Function type definitions
 * -------------------------------------------------------------------*/
typedef XVisualInfo* (*PFNPROXYCHOOSEVISUALPROC)( Display *dpy, int screen, int *attribList );
typedef GLXContext (*PFNPROXYCREATECONTEXTPROC)( Display *dpy, XVisualInfo *vis, GLXContext shareList, Bool direct );
typedef void (*PFNPROXYDESTROYCONTEXTPROC)( Display *dpy, GLXContext ctx );
typedef Bool (*PFNPROXYMAKECURRENTPROC)( Display *dpy, GLXDrawable drawable, GLXContext ctx);
typedef void (*PFNPROXYCOPYCONTEXTPROC)( Display *dpy, GLXContext src, GLXContext dst, unsigned long mask );
typedef void (*PFNPROXYSWAPBUFFERSPROC)( Display *dpy, GLXDrawable drawable );
typedef GLXPixmap (*PFNPROXYCREATEGLXPIXMAPPROC)( Display *dpy, XVisualInfo *visual, Pixmap pixmap );
typedef void (*PFNPROXYDESTROYGLXPIXMAPPROC)( Display *dpy, GLXPixmap pixmap );
typedef Bool (*PFNPROXYQUERYEXTENSIONPROC)( Display *dpy, int *errorb, int *event );
typedef Bool (*PFNPROXYQUERYVERSIONPROC)( Display *dpy, int *maj, int *min );
typedef Bool (*PFNPROXYISDIRECTPROC)( Display *dpy, GLXContext ctx );
typedef int (*PFNPROXYGETCONFIGPROC)( Display *dpy, XVisualInfo *visual, int attrib, int *value );
typedef GLXContext (*PFNPROXYGETCURRENTCONTEXTPROC)( void );
typedef GLXDrawable (*PFNPROXYGETCURRENTDRAWABLEPROC)( void );
typedef void (*PFNPROXYWAITGLPROC)( void );
typedef void (*PFNPROXYWAITXPROC)( void );
typedef void (*PFNPROXYUSEXFONTPROC)( Font font, int first, int count, int list );

/* GLX 1.1 and later */
typedef const char * (*PFNPROXYQUERYEXTENSIONSSTRINGPROC)( Display *dpy, int screen );
typedef const char * (*PFNPROXYQUERYSERVERSTRINGPROC)( Display *dpy, int screen, int name );
typedef const char * (*PFNPROXYGETCLIENTSTRINGPROC)( Display *dpy, int name );

/* GLX 1.2 and later */
typedef Display * (*PFNPROXYGETCURRENTDISPLAYPROC)( void );

/* GLX 1.3 and later */
typedef GLXFBConfig * (*PFNPROXYCHOOSEFBCONFIGPROC)( Display *dpy, int screen, const int *attribList, int *nitems );
typedef int (*PFNPROXYGETFBCONFIGATTRIBPROC)( Display *dpy, GLXFBConfig config, int attribute, int *value );
typedef GLXFBConfig * (*PFNPROXYGETFBCONFIGSPROC)( Display *dpy, int screen, int *nelements );
typedef XVisualInfo * (*PFNPROXYGETVISUALFROMFBCONFIGPROC)( Display *dpy, GLXFBConfig config );
typedef GLXWindow (*PFNPROXYCREATEWINDOWPROC)( Display *dpy, GLXFBConfig config, Window win, const int *attribList );
typedef void (*PFNPROXYDESTROYWINDOWPROC)( Display *dpy, GLXWindow window );
typedef GLXPixmap (*PFNPROXYCREATEPIXMAPPROC)( Display *dpy, GLXFBConfig config, Pixmap pixmap, const int *attribList );
typedef void (*PFNPROXYDESTROYPIXMAPPROC)( Display *dpy, GLXPixmap pixmap );
typedef GLXPbuffer (*PFNPROXYCREATEPBUFFERPROC)( Display *dpy, GLXFBConfig config, const int *attribList );
typedef void (*PFNPROXYDESTROYPBUFFERPROC)( Display *dpy, GLXPbuffer pbuf );
typedef void (*PFNPROXYQUERYDRAWABLEPROC)( Display *dpy, GLXDrawable draw, int attribute, unsigned int *value );
typedef GLXContext (*PFNPROXYCREATENEWCONTEXTPROC)( Display *dpy, GLXFBConfig config, int renderType, GLXContext shareList, Bool direct );
typedef Bool (*PFNPROXYMAKECONTEXTCURRENTPROC)( Display *dpy, GLXDrawable draw, GLXDrawable read, GLXContext ctx );
typedef GLXDrawable (*PFNPROXYGETCURRENTREADDRAWABLEPROC)( void );
typedef int (*PFNPROXYQUERYCONTEXTPROC)( Display *dpy, GLXContext ctx, int attribute, int *value );
typedef void (*PFNPROXYSELECTEVENTPROC)( Display *dpy, GLXDrawable drawable, unsigned long mask );
typedef void (*PFNPROXYGETSELECTEDEVENTPROC)( Display *dpy, GLXDrawable drawable, unsigned long *mask );

#ifndef __GLXextFuncPtr
typedef void (*__GLXextFuncPtr)(void);
#endif
typedef __GLXextFuncPtr (*PFNPROXYGLXGETPROCADDRESSPROC)( const GLubyte *procName);

/* ---------------------------------------------------------------------------------
 * Function rename
 * -------------------------------------------------------------------------------*/
#define glXChooseVisual              pfnProxyChooseVisual
#define glXCreateContext             pfnProxyCreateContext
#define glXDestroyContext            pfnProxyDestroyContext
#define glXMakeCurrent               pfnProxyMakeCurrent
#define glXCopyContext               pfnProxyCopyContext
#define glXSwapBuffers               pfnProxySwapBuffers
#define glXCreateGLXPixmap           pfnProxyCreateGLXPixmap
#define glXDestroyGLXPixmap          pfnProxyDestroyGLXPixmap
#define glXQueryExtension            pfnProxyQueryExtension
#define glXQueryVersion              pfnProxyQueryVersion
#define glXIsDirect                  pfnProxyIsDirect
#define glXGetConfig                 pfnProxyGetConfig
#define glXGetCurrentContext         pfnProxyGetCurrentContext
#define glXGetCurrentDrawable        pfnProxyGetCurrentDrawable
#define glXWaitGL                    pfnProxyWaitGL
#define glXWaitX                     pfnProxyWaitX
#define glXUseXFont                  pfnProxyUseXFont
#define glXQueryExtensionsString     pfnProxyQueryExtensionsString
#define glXQueryServerString         pfnProxyQueryServerString
#define glXGetClientString           pfnProxyGetClientString
#define glXGetCurrentDisplay         pfnProxyGetCurrentDisplay
#define glXChooseFBConfig            pfnProxyChooseFBConfig
#define glXGetFBConfigAttrib         pfnProxyGetFBConfigAttrib
#define glXGetFBConfigs              pfnProxyGetFBConfigs
#define glXGetVisualFromFBConfig     pfnProxyGetVisualFromFBConfig
#define glXCreateWindow              pfnProxyCreateWindow
#define glXDestroyWindow             pfnProxyDestroyWindow
#define glXCreatePixmap              pfnProxyCreatePixmap
#define glXDestroyPixmap             pfnProxyDestroyPixmap
#define glXCreatePBuffer             pfnProxyCreatePBuffer
#define glXDestroyPBuffer            pfnProxyDestroyPBuffer
#define glXQueryDrawable             pfnProxyQueryDrawable
#define glXCreateNewContext          pfnProxyCreateNewContext
#define glXMakeContextCurrent        pfnProxyMakeContextCurrent
#define glXGetCurrentReadDrawable    pfnProxyGetCurrentReadDrawable
#define glXQueryContext              pfnProxyQueryContext
#define glXSelectEvent               pfnProxySelectEvent
#define glXSelectedEvent             pfnProxySelectedEvent
#define glXGetProcAddressARB         pfnProxyGLXGetProcAddressARB
#define glXGetProcAddress            pfnProxyGLXGetProcAddress

/* --------------------------------------------------------------------------
 * export proxy functions
 * ------------------------------------------------------------------------*/
extern PFNPROXYCHOOSEVISUALPROC            pfnProxyChooseVisual;
extern PFNPROXYCREATECONTEXTPROC           pfnProxyCreateContext;
extern PFNPROXYDESTROYCONTEXTPROC          pfnProxyDestroyContext;
extern PFNPROXYMAKECURRENTPROC             pfnProxyMakeCurrent;
extern PFNPROXYCOPYCONTEXTPROC             pfnProxyCopyContext;
extern PFNPROXYSWAPBUFFERSPROC             pfnProxySwapBuffers;
extern PFNPROXYCREATEGLXPIXMAPPROC         pfnProxyCreateGLXPixmap;
extern PFNPROXYDESTROYGLXPIXMAPPROC        pfnProxyDestroyGLXPixmap;
extern PFNPROXYQUERYEXTENSIONPROC          pfnProxyQueryExtension;
extern PFNPROXYQUERYVERSIONPROC            pfnProxyQueryVersion;
extern PFNPROXYISDIRECTPROC                pfnProxyIsDirect;
extern PFNPROXYGETCONFIGPROC               pfnProxyGetConfig;
extern PFNPROXYGETCURRENTCONTEXTPROC       pfnProxyGetCurrentContext;
extern PFNPROXYGETCURRENTDRAWABLEPROC      pfnProxyGetCurrentDrawable;
extern PFNPROXYWAITGLPROC                  pfnProxyWaitGL;
extern PFNPROXYWAITXPROC                   pfnProxyWaitX;
extern PFNPROXYUSEXFONTPROC                pfnProxyUseXFont;
extern PFNPROXYQUERYEXTENSIONSSTRINGPROC   pfnProxyQueryExtensionsString;
extern PFNPROXYQUERYSERVERSTRINGPROC       pfnProxyQueryServerString;
extern PFNPROXYGETCLIENTSTRINGPROC         pfnProxyGetClientString;
extern PFNPROXYGETCURRENTDISPLAYPROC       pfnProxyGetCurrentDisplay;
extern PFNPROXYCHOOSEFBCONFIGPROC          pfnProxyChooseFBConfig;
extern PFNPROXYGETFBCONFIGATTRIBPROC       pfnProxyGetFBConfigAttrib;
extern PFNPROXYGETFBCONFIGSPROC            pfnProxyGetFBConfigs;
extern PFNPROXYGETVISUALFROMFBCONFIGPROC   pfnProxyGetVisualFromFBConfig;
extern PFNPROXYCREATEWINDOWPROC            pfnProxyCreateWindow;
extern PFNPROXYDESTROYWINDOWPROC           pfnProxyDestroyWindow;
extern PFNPROXYCREATEPIXMAPPROC            pfnProxyCreatePixmap;
extern PFNPROXYDESTROYPIXMAPPROC           pfnProxyDestroyPixmap;
extern PFNPROXYCREATEPBUFFERPROC           pfnProxyCreatePBuffer;
extern PFNPROXYDESTROYPBUFFERPROC          pfnProxyDestroyPBuffer;
extern PFNPROXYQUERYDRAWABLEPROC           pfnProxyQueryDrawable;
extern PFNPROXYCREATENEWCONTEXTPROC        pfnProxyCreateNewContext;
extern PFNPROXYMAKECONTEXTCURRENTPROC      pfnProxyMakeContextCurrent;
extern PFNPROXYGETCURRENTREADDRAWABLEPROC  pfnProxyGetCurrentReadDrawable;
extern PFNPROXYQUERYCONTEXTPROC            pfnProxyQueryContext;
extern PFNPROXYSELECTEVENTPROC             pfnProxySelectEvent;
extern PFNPROXYGETSELECTEDEVENTPROC        pfnProxySelectedEvent;
extern PFNPROXYGLXGETPROCADDRESSPROC       pfnProxyGLXGetProcAddressARB;
extern PFNPROXYGLXGETPROCADDRESSPROC       pfnProxyGLXGetProcAddress;

#ifdef __cplusplus
}
#endif

#endif
