/* Copyright (C) 2010-2011 Linaro Ltd.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

/*
 * Authored-by: Jammy Zhou <jammy.zhou@linaro.org>
 */

#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <dlfcn.h>
#include <string.h>
#include <GL/gl_proxy.h>
#include <GL/glx_proxy.h>

#include "egldef.h"

/* global proxy context */
PROXY_CONTEXT global_proxy_context;

/* library names for different backends */
const char *opengl_libraries[] = {
	"libGL.so",
	"libGLESv2.so",
};

/* proxy functions */
PFNPROXYACTIVETEXTUREPROC pfnProxyActiveTexture = NULL;
PFNPROXYATTACHSHADERPROC pfnProxyAttachShader = NULL;
PFNPROXYBINDATTRIBLOCATIONPROC pfnProxyBindAttribLocation = NULL;
PFNPROXYBINDBUFFERPROC pfnProxyBindBuffer = NULL;
PFNPROXYBINDFRAMEBUFFERPROC pfnProxyBindFramebuffer = NULL;
PFNPROXYBINDRENDERBUFFERPROC pfnProxyBindRenderbuffer = NULL;
PFNPROXYBINDTEXTUREPROC pfnProxyBindTexture = NULL;
PFNPROXYBLENDCOLORPROC pfnProxyBlendColor = NULL;
PFNPROXYBLENDEQUATIONPROC pfnProxyBlendEquation = NULL;
PFNPROXYBLENDEQUATIONSEPARATEPROC pfnProxyBlendEquationSeparate = NULL;
PFNPROXYBLENDFUNCPROC pfnProxyBlendFunc = NULL;
PFNPROXYBLENDFUNCSEPARATEPROC pfnProxyBlendFuncSeparate = NULL;
PFNPROXYBUFFERDATAPROC pfnProxyBufferData = NULL;
PFNPROXYBUFFERSUBDATAPROC pfnProxyBufferSubData = NULL;
PFNPROXYCHECKFRAMEBUFFERSTATUSPROC pfnProxyCheckFramebufferStatus = NULL;
PFNPROXYCLEARPROC pfnProxyClear = NULL;
PFNPROXYCLEARCOLORPROC pfnProxyClearColor = NULL;
PFNPROXYCLEARDEPTHFPROC pfnProxyClearDepthf = NULL;
PFNPROXYCLEARSTENCILPROC pfnProxyClearStencil = NULL;
PFNPROXYCOLORMASKPROC pfnProxyColorMask = NULL;
PFNPROXYCOMPILESHADERPROC pfnProxyCompileShader = NULL;
PFNPROXYCOMPRESSEDTEXIMAGE2DPROC pfnProxyCompressedTexImage2D = NULL;
PFNPROXYCOMPRESSEDTEXSUBIMAGE2DPROC pfnProxyCompressedTexSubImage2D = NULL;
PFNPROXYCOPYTEXIMAGE2DPROC pfnProxyCopyTexImage2D = NULL;
PFNPROXYCOPYTEXSUBIMAGE2DPROC pfnProxyCopyTexSubImage2D = NULL;
PFNPROXYCREATEPROGRAMPROC pfnProxyCreateProgram = NULL;
PFNPROXYCREATESHADERPROC pfnProxyCreateShader = NULL;
PFNPROXYCULLFACEPROC pfnProxyCullFace = NULL;
PFNPROXYDELETEBUFFERSPROC pfnProxyDeleteBuffers = NULL;
PFNPROXYDELETEFRAMEBUFFERSPROC pfnProxyDeleteFramebuffers = NULL;
PFNPROXYDELETEPROGRAMPROC pfnProxyDeleteProgram = NULL;
PFNPROXYDELETERENDERBUFFERSPROC pfnProxyDeleteRenderbuffers = NULL;
PFNPROXYDELETESHADERPROC pfnProxyDeleteShader = NULL;
PFNPROXYDELETETEXTURESPROC pfnProxyDeleteTextures = NULL;
PFNPROXYDEPTHFUNCPROC pfnProxyDepthFunc = NULL;
PFNPROXYDEPTHMASKPROC pfnProxyDepthMask = NULL;
PFNPROXYDEPTHRANGEFPROC pfnProxyDepthRangef = NULL;
PFNPROXYDETACHSHADERPROC pfnProxyDetachShader = NULL;
PFNPROXYDISABLEPROC pfnProxyDisable = NULL;
PFNPROXYDISABLEVERTEXATTRIBARRAYPROC pfnProxyDisableVertexAttribArray = NULL;
PFNPROXYDRAWARRAYSPROC pfnProxyDrawArrays = NULL;
PFNPROXYDRAWELEMENTSPROC pfnProxyDrawElements = NULL;
PFNPROXYENABLEPROC pfnProxyEnable = NULL;
PFNPROXYENABLEVERTEXATTRIBARRAYPROC pfnProxyEnableVertexAttribArray = NULL;
PFNPROXYFINISHPROC pfnProxyFinish = NULL;
PFNPROXYFLUSHPROC pfnProxyFlush = NULL;
PFNPROXYFRAMEBUFFERRENDERBUFFERPROC pfnProxyFramebufferRenderbuffer = NULL;
PFNPROXYFRAMEBUFFERTEXTURE2DPROC pfnProxyFramebufferTexture2D = NULL;
PFNPROXYFRONTFACEPROC pfnProxyFrontFace = NULL;
PFNPROXYGENBUFFERSPROC pfnProxyGenBuffers = NULL;
PFNPROXYGENERATEMIPMAPPROC pfnProxyGenerateMipmap = NULL;
PFNPROXYGENFRAMEBUFFERSPROC pfnProxyGenFramebuffers = NULL;
PFNPROXYGENRENDERBUFFERSPROC pfnProxyGenRenderbuffers = NULL;
PFNPROXYGENTEXTURESPROC pfnProxyGenTextures = NULL;
PFNPROXYGETACTIVEATTRIBPROC pfnProxyGetActiveAttrib = NULL;
PFNPROXYGETACTIVEUNIFORMPROC pfnProxyGetActiveUniform = NULL;
PFNPROXYGETATTACHEDSHADERSPROC pfnProxyGetAttachedShaders = NULL;
PFNPROXYGETATTRIBLOCATIONPROC pfnProxyGetAttribLocation = NULL;
PFNPROXYGETBOOLEANVPROC pfnProxyGetBooleanv = NULL;
PFNPROXYGETBUFFERPARAMETERIVPROC pfnProxyGetBufferParameteriv = NULL;
PFNPROXYGETERRORPROC pfnProxyGetError = NULL;
PFNPROXYGETFLOATVPROC pfnProxyGetFloatv = NULL;
PFNPROXYGETFRAMEBUFFERATTACHMENTPARAMETERIVPROC pfnProxyGetFramebufferAttachmentParameteriv = NULL;
PFNPROXYGETINTEGERVPROC pfnProxyGetIntegerv = NULL;
PFNPROXYGETPROGRAMIVPROC pfnProxyGetProgramiv = NULL;
PFNPROXYGETPROGRAMINFOLOGPROC pfnProxyGetProgramInfoLog = NULL;
PFNPROXYGETRENDERBUFFERPARAMETERIVPROC pfnProxyGetRenderbufferParameteriv = NULL;
PFNPROXYGETSHADERIVPROC pfnProxyGetShaderiv = NULL;
PFNPROXYGETSHADERINFOLOGPROC pfnProxyGetShaderInfoLog = NULL;
PFNPROXYGETSHADERPRECISIONFORMATPROC pfnProxyGetShaderPrecisionFormat = NULL;
PFNPROXYGETSHADERSOURCEPROC pfnProxyGetShaderSource = NULL;
PFNPROXYGETSTRINGPROC pfnProxyGetString = NULL;
PFNPROXYGETTEXPARAMETERFVPROC pfnProxyGetTexParameterfv = NULL;
PFNPROXYGETTEXPARAMETERIVPROC pfnProxyGetTexParameteriv = NULL;
PFNPROXYGETUNIFORMFVPROC pfnProxyGetUniformfv = NULL;
PFNPROXYGETUNIFORMIVPROC pfnProxyGetUniformiv = NULL;
PFNPROXYGETUNIFORMLOCATIONPROC pfnProxyGetUniformLocation = NULL;
PFNPROXYGETVERTEXATTRIBFVPROC pfnProxyGetVertexAttribfv = NULL;
PFNPROXYGETVERTEXATTRIBIVPROC pfnProxyGetVertexAttribiv = NULL;
PFNPROXYGETVERTEXATTRIBPOINTERVPROC pfnProxyGetVertexAttribPointerv = NULL;
PFNPROXYHINTPROC pfnProxyHint = NULL;
PFNPROXYISBUFFERPROC pfnProxyIsBuffer = NULL;
PFNPROXYISENABLEDPROC pfnProxyIsEnabled = NULL;
PFNPROXYISFRAMEBUFFERPROC pfnProxyIsFramebuffer = NULL;
PFNPROXYISPROGRAMPROC pfnProxyIsProgram = NULL;
PFNPROXYISRENDERBUFFERPROC pfnProxyIsRenderbuffer = NULL;
PFNPROXYISSHADERPROC pfnProxyIsShader = NULL;
PFNPROXYISTEXTUREPROC pfnProxyIsTexture = NULL;
PFNPROXYLINEWIDTHPROC pfnProxyLineWidth = NULL;
PFNPROXYLINKPROGRAMPROC pfnProxyLinkProgram = NULL;
PFNPROXYPIXELSTOREIPROC pfnProxyPixelStorei = NULL;
PFNPROXYPOLYGONOFFSETPROC pfnProxyPolygonOffset = NULL;
PFNPROXYREADPIXELSPROC pfnProxyReadPixels = NULL;
PFNPROXYRELEASESHADERCOMPILERPROC pfnProxyReleaseShaderCompiler = NULL;
PFNPROXYRENDERBUFFERSTORAGEPROC pfnProxyRenderbufferStorage = NULL;
PFNPROXYSAMPLECOVERAGEPROC pfnProxySampleCoverage = NULL;
PFNPROXYSCISSORPROC pfnProxyScissor = NULL;
PFNPROXYSHADERBINARYPROC pfnProxyShaderBinary = NULL;
PFNPROXYSHADERSOURCEPROC pfnProxyShaderSource = NULL;
PFNPROXYSTENCILFUNCPROC pfnProxyStencilFunc = NULL;
PFNPROXYSTENCILFUNCSEPARATEPROC pfnProxyStencilFuncSeparate = NULL;
PFNPROXYSTENCILMASKPROC pfnProxyStencilMask = NULL;
PFNPROXYSTENCILMASKSEPARATEPROC pfnProxyStencilMaskSeparate = NULL;
PFNPROXYSTENCILOPPROC pfnProxyStencilOp = NULL;
PFNPROXYSTENCILOPSEPARATEPROC pfnProxyStencilOpSeparate = NULL;
PFNPROXYTEXIMAGE2DPROC pfnProxyTexImage2D = NULL;
PFNPROXYTEXPARAMETERFPROC pfnProxyTexParameterf = NULL;
PFNPROXYTEXPARAMETERFVPROC pfnProxyTexParameterfv = NULL;
PFNPROXYTEXPARAMETERIPROC pfnProxyTexParameteri = NULL;
PFNPROXYTEXPARAMETERIVPROC pfnProxyTexParameteriv = NULL;
PFNPROXYTEXSUBIMAGE2DPROC pfnProxyTexSubImage2D = NULL;
PFNPROXYUNIFORM1FPROC pfnProxyUniform1f = NULL;
PFNPROXYUNIFORM1FVPROC pfnProxyUniform1fv = NULL;
PFNPROXYUNIFORM1IPROC pfnProxyUniform1i = NULL;
PFNPROXYUNIFORM1IVPROC pfnProxyUniform1iv = NULL;
PFNPROXYUNIFORM2FPROC pfnProxyUniform2f = NULL;
PFNPROXYUNIFORM2FVPROC pfnProxyUniform2fv = NULL;
PFNPROXYUNIFORM2IPROC pfnProxyUniform2i = NULL;
PFNPROXYUNIFORM2IVPROC pfnProxyUniform2iv = NULL;
PFNPROXYUNIFORM3FPROC pfnProxyUniform3f = NULL;
PFNPROXYUNIFORM3FVPROC pfnProxyUniform3fv = NULL;
PFNPROXYUNIFORM3IPROC pfnProxyUniform3i = NULL;
PFNPROXYUNIFORM3IVPROC pfnProxyUniform3iv = NULL;
PFNPROXYUNIFORM4FPROC pfnProxyUniform4f = NULL;
PFNPROXYUNIFORM4FVPROC pfnProxyUniform4fv = NULL;
PFNPROXYUNIFORM4IPROC pfnProxyUniform4i = NULL;
PFNPROXYUNIFORM4IVPROC pfnProxyUniform4iv = NULL;
PFNPROXYUNIFORMMATRIX2FVPROC pfnProxyUniformMatrix2fv = NULL;
PFNPROXYUNIFORMMATRIX3FVPROC pfnProxyUniformMatrix3fv = NULL;
PFNPROXYUNIFORMMATRIX4FVPROC pfnProxyUniformMatrix4fv = NULL;
PFNPROXYUSEPROGRAMPROC pfnProxyUseProgram = NULL;
PFNPROXYVALIDATEPROGRAMPROC pfnProxyValidateProgram = NULL;
PFNPROXYVERTEXATTRIB1FPROC pfnProxyVertexAttrib1f = NULL;
PFNPROXYVERTEXATTRIB1FVPROC pfnProxyVertexAttrib1fv = NULL;
PFNPROXYVERTEXATTRIB2FPROC pfnProxyVertexAttrib2f = NULL;
PFNPROXYVERTEXATTRIB2FVPROC pfnProxyVertexAttrib2fv = NULL;
PFNPROXYVERTEXATTRIB3FPROC pfnProxyVertexAttrib3f = NULL;
PFNPROXYVERTEXATTRIB3FVPROC pfnProxyVertexAttrib3fv = NULL;
PFNPROXYVERTEXATTRIB4FPROC pfnProxyVertexAttrib4f = NULL;
PFNPROXYVERTEXATTRIB4FVPROC pfnProxyVertexAttrib4fv = NULL;
PFNPROXYVERTEXATTRIBPOINTERPROC pfnProxyVertexAttribPointer = NULL;
PFNPROXYVIEWPORTPROC pfnProxyViewport = NULL;

/* Proc address for EGL/GLX symbol resolve function
 * Note:
 * eglGetProcAddress can only be used for GL or EGL extension function
 * glXGetProcAddress/glXGetProcAddressARB can be used for all GL/GLX funtions
 */
PFNPROXYGLXGETPROCADDRESSPROC pfnProxyGLXGetProcAddressARB = NULL;
PFNPROXYGLXGETPROCADDRESSPROC pfnProxyGLXGetProcAddress = NULL;
PFNPROXYEGLGETPROCADDRESSPROC pfnProxyEGLGetProcAddress = NULL;

/* EGL APIs needed for backend selection */
PFNPROXYEGLGETDISPLAYPROC pfnProxyEGLGetDisplay = NULL;
PFNPROXYEGLINITIALIZEPROC pfnProxyEGLInitialize = NULL;
PFNPROXYEGLTERMINATEPROC pfnProxyEGLTerminate = NULL;
PFNPROXYEGLGETCONFIGSPROC pfnProxyEGLGetConfigs = NULL;
PFNPROXYEGLCREATECONTEXTPROC pfnProxyEGLCreateContext = NULL;
PFNPROXYEGLDESTROYCONTEXTPROC pfnProxyEGLDestroyContext = NULL;
PFNPROXYEGLCREATEWINDOWSURFACEPROC pfnProxyEGLCreateWindowSurface = NULL;
PFNPROXYEGLDESTROYSURFACEPROC pfnProxyEGLDestroySurface = NULL;
PFNPROXYEGLMAKECURRENTPROC pfnProxyEGLMakeCurrent = NULL;

static void *glProxyOpenLibrary(const char *name)
{
	return dlopen(name, RTLD_LAZY | RTLD_GLOBAL);
}

static int glProxyCloseLibrary(void *handle)
{
	return dlclose(handle);
}

/* A valid EGL rendering context is needed in scenarios similar as below:
 * - call glGetString to get renderer information
 * - call eglGetProcAddress to get symbol address for functions of GLES2 extenstions
 */
static int CreateEGLRenderingContext(Display *dpy, Window win, PROXY_CONTEXT *proxy_context)
{
	EGLDisplay egl_dpy;
	EGLConfig config;
	EGLint num_configs = 1;

	static const EGLint ctx_attribs[] = {
		EGL_CONTEXT_CLIENT_VERSION, 2,
		EGL_NONE
	};

	if (proxy_context->egl_dpy && proxy_context->egl_ctx && proxy_context->egl_surf)
		return 0;

	egl_dpy = proxy_context->egl_dpy = (*pfnProxyEGLGetDisplay)(dpy);

	if ((EGL_NO_DISPLAY == egl_dpy) || !(*pfnProxyEGLInitialize)(egl_dpy, NULL, NULL)) {
		printf("egl initialization failed\n");
		return -1;
	}
        
	if (!(*pfnProxyEGLGetConfigs)(egl_dpy, &config, num_configs, &num_configs) || num_configs < 1) {
		printf("failed to get egl config\n");
		return -1;
	}
        
	proxy_context->egl_ctx = (*pfnProxyEGLCreateContext)(egl_dpy, config, EGL_NO_CONTEXT, ctx_attribs);
	if (EGL_NO_CONTEXT == proxy_context->egl_ctx) {
		printf("eglCreateContext failed\n");
		return -1;
	}
        
	proxy_context->egl_surf = (*pfnProxyEGLCreateWindowSurface)(egl_dpy, config, win, NULL);
	if (EGL_NO_SURFACE == proxy_context->egl_surf) {
		printf("eglCreateWindowSurface failed\n");
		return -1;
	}
        
	if (!(*pfnProxyEGLMakeCurrent)(egl_dpy, proxy_context->egl_surf, proxy_context->egl_surf, proxy_context->egl_ctx)) {
		printf("eglMakeCurrent failed\n");
		return -1;
	}

	return 0;
}

static int DestroyEGLRenderingContext(PROXY_CONTEXT *proxy_context)
{
	(*pfnProxyEGLMakeCurrent)(proxy_context->egl_dpy, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT);
	(*pfnProxyEGLDestroyContext)(proxy_context->egl_dpy, proxy_context->egl_ctx);
	(*pfnProxyEGLDestroySurface)(proxy_context->egl_dpy, proxy_context->egl_surf);
	(*pfnProxyEGLTerminate)(proxy_context->egl_dpy);

	proxy_context->egl_dpy = EGL_NO_DISPLAY;
	proxy_context->egl_ctx = EGL_NO_CONTEXT;
	proxy_context->egl_surf = EGL_NO_SURFACE;

	return 0;
}

static int glProxyEarlyResolve(PROXY_CONTEXT *proxy_context)
{
	if (OPENGL_BACKEND == proxy_context->current_backend) {
		proxy_context->gl_handle = glProxyOpenLibrary(opengl_libraries[OPENGL_BACKEND]);
		pfnProxyGLXGetProcAddress = dlsym(proxy_context->gl_handle, "glXGetProcAddress");
		if (NULL == pfnProxyGLXGetProcAddress) {
			pfnProxyGLXGetProcAddressARB = dlsym(proxy_context->gl_handle, "glXGetProcAddressARB");
		}
	}
	else if (OPENGL_ES20_BACKEND == proxy_context->current_backend) {
		Display *dpy = XOpenDisplay(NULL);
		int scrnum;
		Window root;

		if (NULL == dpy) {
			printf("failed to open display\n");
			return -1;
		}

		scrnum = DefaultScreen(dpy);
		root = RootWindow(dpy, scrnum);

		if (NULL == proxy_context->egl_handle) {
			proxy_context->egl_handle = glProxyOpenLibrary(LIBEGL_PATH);
			pfnProxyEGLGetDisplay = dlsym(proxy_context->egl_handle, "eglGetDisplay");
			pfnProxyEGLInitialize = dlsym(proxy_context->egl_handle, "eglInitialize");
			pfnProxyEGLGetConfigs = dlsym(proxy_context->egl_handle, "eglGetConfigs");
			pfnProxyEGLCreateContext = dlsym(proxy_context->egl_handle, "eglCreateContext");
			pfnProxyEGLCreateWindowSurface = dlsym(proxy_context->egl_handle, "eglCreateWindowSurface");
			pfnProxyEGLMakeCurrent = dlsym(proxy_context->egl_handle, "eglMakeCurrent");
			pfnProxyEGLDestroyContext = dlsym(proxy_context->egl_handle, "eglDestroyContext");
			pfnProxyEGLDestroySurface = dlsym(proxy_context->egl_handle, "eglDestroySurface");
			pfnProxyEGLTerminate = dlsym(proxy_context->egl_handle, "eglTerminate");
		}

		if (CreateEGLRenderingContext(dpy, root, proxy_context) < 0)
			return -1;

		pfnProxyEGLGetProcAddress = dlsym(proxy_context->egl_handle, "eglGetProcAddress");
	}

	return 0;
}

void *glProxyGetProcAddr(void *handle, const char *name)
{
	if (pfnProxyGLXGetProcAddress) {
		return (*pfnProxyGLXGetProcAddress)(name);
	}
	else if (pfnProxyGLXGetProcAddressARB) {
		return (*pfnProxyGLXGetProcAddressARB)(name);
	}
	else {
		return dlsym(handle, name);
	}
}

void *glProxyGetExtProcAddr(void *handle, const char *name)
{
	if (pfnProxyEGLGetProcAddress) {
		return (*pfnProxyEGLGetProcAddress)(name);
	}
	else if (pfnProxyGLXGetProcAddress) {
 		return (*pfnProxyGLXGetProcAddress)(name);
 	}
	else if (pfnProxyGLXGetProcAddressARB) {
		return (*pfnProxyGLXGetProcAddressARB)(name);
	}
	else {
		return dlsym(handle, name);
	}
}

static const char *glProxyGetGLRenderer(Display *dpy)
{
	int scrnum;
	Window root;
	const char *renderer = NULL;

	GLXContext glx_ctx;
	int attribs[] = {
		GLX_RGBA,
		None };
	XVisualInfo *visinfo = NULL;

	scrnum = DefaultScreen(dpy);
        root = RootWindow(dpy, scrnum);

	visinfo = (*pfnProxyChooseVisual)(dpy, scrnum, attribs);

	if (!visinfo) {
 		printf("failed to get visual\n");
		return NULL;
	}

	glx_ctx = (*pfnProxyCreateContext)(dpy, visinfo, NULL, True);
	if (!glx_ctx) {
		printf("failed to create GLX context\n");
		return NULL;
	}

	(*pfnProxyMakeCurrent)(dpy, root, glx_ctx);

	renderer = (*pfnProxyGetString)(GL_RENDERER);

	(*pfnProxyMakeCurrent)(dpy, None, NULL);

	(*pfnProxyDestroyContext)(dpy, glx_ctx);

	XFree(visinfo);

	return renderer;
}

static void glProxyBackendSelection(PROXY_BACKEND_OPTION *pProxyBackend, PROXY_CONTEXT *proxy_context)
{
	PROXY_BACKEND_OPTION option = *pProxyBackend;
	const char *gl_renderer = NULL;
	const char *gles2_renderer = NULL;

	if (OPENGL_BACKEND == option) {
                if (proxy_context->gl_handle = glProxyOpenLibrary(opengl_libraries[OPENGL_BACKEND])) {
                        *pProxyBackend = OPENGL_BACKEND;
                }
                else if (proxy_context->gles2_handle = glProxyOpenLibrary(opengl_libraries[OPENGL_ES20_BACKEND])) {
                        *pProxyBackend = OPENGL_ES20_BACKEND;
                }
                else {
                        *pProxyBackend = UNKNOWN_BACKEND;
                }
        } else if (OPENGL_ES20_BACKEND == option) {
                if (proxy_context->gles2_handle = glProxyOpenLibrary(opengl_libraries[OPENGL_ES20_BACKEND])) {
                        *pProxyBackend = OPENGL_ES20_BACKEND;
                }
                else if (proxy_context->gl_handle = glProxyOpenLibrary(opengl_libraries[OPENGL_BACKEND])) {
                        *pProxyBackend = OPENGL_BACKEND;
                }
                else {
                        *pProxyBackend = UNKNOWN_BACKEND;
                }
        } else {
                /* Select backend automatically */
		proxy_context->gles2_handle = glProxyOpenLibrary(opengl_libraries[OPENGL_ES20_BACKEND]);
                proxy_context->egl_handle = glProxyOpenLibrary(LIBEGL_PATH);
		proxy_context->gl_handle = glProxyOpenLibrary(opengl_libraries[OPENGL_BACKEND]);

#if 1 
		/* There is a bug for mesa driver to get GL and GLES2 renderer info in one process
		 * https://bugs.freedesktop.org/show_bug.cgi?id=32285
		 */

		Display *dpy = XOpenDisplay(NULL);

		if (NULL == dpy) {
			printf("failed to open display\n");
			*pProxyBackend = UNKNOWN_BACKEND;
			return;
		}

		proxy_context->x_dpy = dpy;

		/* Try to get the gles2 renderer info */
		if (proxy_context->egl_handle && proxy_context->gles2_handle) {
			int scrnum;
			Window root;

			scrnum = DefaultScreen(dpy);
			root = RootWindow(dpy, scrnum);

			/* Resolve EGL APIs for create rendering context to get GL_RENDERER */
                	pfnProxyEGLGetDisplay = dlsym(proxy_context->egl_handle, "eglGetDisplay");
                	pfnProxyEGLInitialize = dlsym(proxy_context->egl_handle, "eglInitialize");
                	pfnProxyEGLGetConfigs = dlsym(proxy_context->egl_handle, "eglGetConfigs");
                	pfnProxyEGLCreateContext = dlsym(proxy_context->egl_handle, "eglCreateContext");
                	pfnProxyEGLCreateWindowSurface = dlsym(proxy_context->egl_handle, "eglCreateWindowSurface");
                	pfnProxyEGLMakeCurrent = dlsym(proxy_context->egl_handle, "eglMakeCurrent");
                	pfnProxyEGLDestroyContext = dlsym(proxy_context->egl_handle, "eglDestroyContext");
                	pfnProxyEGLDestroySurface = dlsym(proxy_context->egl_handle, "eglDestroySurface");
                	pfnProxyEGLTerminate = dlsym(proxy_context->egl_handle, "eglTerminate");

			/* Resolve glGetString */
			pfnProxyGetString = dlsym(proxy_context->gles2_handle, "glGetString");

			CreateEGLRenderingContext(dpy, root, proxy_context);

			gles2_renderer = pfnProxyGetString(GL_RENDERER);

			/* Reset the function pointers */
			pfnProxyGetString = NULL;
		}

		printf("OpenGL ES2.0 Renderer: %s\n", gles2_renderer ? gles2_renderer : "Not Available");

		/* Try to get the opengl renderer info if available */
		if (proxy_context->gl_handle) {
			/* Resolve GLX APIs needed for create rendering context */
                	pfnProxyChooseVisual = dlsym(proxy_context->gl_handle, "glXChooseVisual");
                	pfnProxyCreateContext = dlsym(proxy_context->gl_handle, "glXCreateContext");
                	pfnProxyMakeCurrent = dlsym(proxy_context->gl_handle, "glXMakeCurrent");
                	pfnProxyDestroyContext = dlsym(proxy_context->gl_handle, "glXDestroyContext");

			/* Resolve glGetString */
                	pfnProxyGetString = dlsym(proxy_context->gl_handle, "glGetString");

			gl_renderer = glProxyGetGLRenderer(dpy);

			/* Reset the function pointers */
			pfnProxyGetString = NULL;
			pfnProxyChooseVisual = NULL;
			pfnProxyCreateContext = NULL;
			pfnProxyMakeCurrent = NULL;
			pfnProxyDestroyContext = NULL;
		}

		printf("OpenGL Renderer: %s\n", gl_renderer ? gl_renderer : "Not Available");

		if (gles2_renderer && gl_renderer) {
			if (strstr(gl_renderer, "Soft") || strstr(gl_renderer, "soft")) {
				/* if OpenGL renderer is SW, use GLES2 renderer */
				*pProxyBackend = OPENGL_ES20_BACKEND;
			}
			else if (strstr(gles2_renderer, "Soft") || strstr(gles2_renderer, "soft")) {
				/* if OpenGL renderer is HW, GLES2 renderer is SW, use OpenGL renderer */
				*pProxyBackend = OPENGL_BACKEND;
			}
			else {
				/* if both renderers are HW, use GLES2 renderer */
				*pProxyBackend = OPENGL_ES20_BACKEND;
			}
		}
		else if (gles2_renderer) {
			*pProxyBackend = OPENGL_ES20_BACKEND;
		}
		else if (gl_renderer) {
			*pProxyBackend = OPENGL_BACKEND;
		}
		else {
			*pProxyBackend = UNKNOWN_BACKEND;
		}
#else
		if (proxy_context->gles2_handle) {
			/* Use GLES2 by default if available */
			*pProxyBackend = OPENGL_ES20_BACKEND;
		}
		else if (proxy_context->gl_handle) {
			*pProxyBackend = OPENGL_BACKEND;
		}
		else {
			*pProxyBackend = UNKNOWN_BACKEND;
		}
		
#endif

		if (OPENGL_ES20_BACKEND == *pProxyBackend) {
			if (proxy_context->gl_handle) {
				glProxyCloseLibrary(proxy_context->gl_handle);
				proxy_context->gl_handle = NULL;
			}
		}
		else if (OPENGL_BACKEND == *pProxyBackend) {
			if (proxy_context->egl_ctx) {
				DestroyEGLRenderingContext(proxy_context);
				pfnProxyEGLGetDisplay = NULL;
				pfnProxyEGLInitialize = NULL;
				pfnProxyEGLGetConfigs = NULL;
				pfnProxyEGLCreateContext = NULL;
				pfnProxyEGLCreateWindowSurface = NULL;
				pfnProxyEGLMakeCurrent = NULL;
				pfnProxyEGLDestroyContext = NULL;
				pfnProxyEGLDestroySurface = NULL;
				pfnProxyEGLTerminate = NULL;
			}

			if (proxy_context->egl_handle) {
				glProxyCloseLibrary(proxy_context->egl_handle);
				proxy_context->egl_handle = NULL;
			}

			if (proxy_context->gles2_handle) {
				glProxyCloseLibrary(proxy_context->gles2_handle);
				proxy_context->gles2_handle = NULL;
			}
		}
	}

	proxy_context->current_backend = *pProxyBackend;
}

static int glProxyCoreInit(void *handle)
{
	if (NULL == handle) {
		return -1;
	}

	pfnProxyActiveTexture = glProxyGetProcAddr(handle, "glActiveTexture");
	pfnProxyAttachShader = glProxyGetProcAddr(handle, "glAttachShader");
	pfnProxyBindAttribLocation = glProxyGetProcAddr(handle, "glBindAttribLocation");
	pfnProxyBindBuffer = glProxyGetProcAddr(handle, "glBindBuffer");
	pfnProxyBindFramebuffer = glProxyGetProcAddr(handle, "glBindFramebuffer");
	pfnProxyBindRenderbuffer = glProxyGetProcAddr(handle, "glBindRenderbuffer");
	pfnProxyBindTexture = glProxyGetProcAddr(handle, "glBindTexture");
	pfnProxyBlendColor = glProxyGetProcAddr(handle, "glBlendColor");
	pfnProxyBlendEquation = glProxyGetProcAddr(handle, "glBlendEquation");
	pfnProxyBlendEquationSeparate = glProxyGetProcAddr(handle, "glBlendEquationSeparate");
	pfnProxyBlendFunc = glProxyGetProcAddr(handle, "glBlendFunc");
	pfnProxyBlendFuncSeparate = glProxyGetProcAddr(handle, "glBlendFuncSeparate");
	pfnProxyBufferData = glProxyGetProcAddr(handle, "glBufferData");
	pfnProxyBufferSubData = glProxyGetProcAddr(handle, "glBufferSubData");
	pfnProxyCheckFramebufferStatus = glProxyGetProcAddr(handle, "glCheckFramebufferStatus");
	pfnProxyClear = glProxyGetProcAddr(handle, "glClear");
	pfnProxyClearColor = glProxyGetProcAddr(handle, "glClearColor");
	pfnProxyClearDepthf = glProxyGetProcAddr(handle, "glClearDepthf");
	pfnProxyClearStencil = glProxyGetProcAddr(handle, "glClearStencil");
	pfnProxyColorMask = glProxyGetProcAddr(handle, "glColorMask");
	pfnProxyCompileShader = glProxyGetProcAddr(handle, "glCompileShader");
	pfnProxyCompressedTexImage2D = glProxyGetProcAddr(handle, "glCompressedTexImage2D");
	pfnProxyCompressedTexSubImage2D = glProxyGetProcAddr(handle, "glCompressedTexSubImage2D");
	pfnProxyCopyTexImage2D = glProxyGetProcAddr(handle, "glCopyTexImage2D");
	pfnProxyCopyTexSubImage2D = glProxyGetProcAddr(handle, "glCopyTexSubImage2D");
	pfnProxyCreateProgram = glProxyGetProcAddr(handle, "glCreateProgram");
	pfnProxyCreateShader = glProxyGetProcAddr(handle, "glCreateShader");
	pfnProxyCullFace = glProxyGetProcAddr(handle, "glCullFace");
	pfnProxyDeleteBuffers = glProxyGetProcAddr(handle, "glDeleteBuffers");
	pfnProxyDeleteFramebuffers = glProxyGetProcAddr(handle, "glDeleteFramebuffers");
	pfnProxyDeleteProgram = glProxyGetProcAddr(handle, "glDeleteProgram");
	pfnProxyDeleteRenderbuffers = glProxyGetProcAddr(handle, "glDeleteRenderbuffers");
	pfnProxyDeleteShader = glProxyGetProcAddr(handle, "glDeleteShader");
	pfnProxyDeleteTextures = glProxyGetProcAddr(handle, "glDeleteTextures");
	pfnProxyDepthFunc = glProxyGetProcAddr(handle, "glDepthFunc");
	pfnProxyDepthMask = glProxyGetProcAddr(handle, "glDepthMask");
	pfnProxyDepthRangef = glProxyGetProcAddr(handle, "glDepthRangef");
	pfnProxyDetachShader = glProxyGetProcAddr(handle, "glDetachShader");
	pfnProxyDisable = glProxyGetProcAddr(handle, "glDisable");
	pfnProxyDisableVertexAttribArray = glProxyGetProcAddr(handle, "glDisableVertexAttribArray");
	pfnProxyDrawArrays = glProxyGetProcAddr(handle, "glDrawArrays");
	pfnProxyDrawElements = glProxyGetProcAddr(handle, "glDrawElements");
	pfnProxyEnable = glProxyGetProcAddr(handle, "glEnable");
	pfnProxyEnableVertexAttribArray = glProxyGetProcAddr(handle, "glEnableVertexAttribArray");
	pfnProxyFinish = glProxyGetProcAddr(handle, "glFinish");
	pfnProxyFlush = glProxyGetProcAddr(handle, "glFlush");
	pfnProxyFramebufferRenderbuffer = glProxyGetProcAddr(handle, "glFramebufferRenderbuffer");
	pfnProxyFramebufferTexture2D = glProxyGetProcAddr(handle, "glFramebufferTexture2D");
	pfnProxyFrontFace = glProxyGetProcAddr(handle, "glFrontFace");
	pfnProxyGenBuffers = glProxyGetProcAddr(handle, "glGenBuffers");
	pfnProxyGenerateMipmap = glProxyGetProcAddr(handle, "glGenerateMipmap");
	pfnProxyGenFramebuffers = glProxyGetProcAddr(handle, "glGenFramebuffers");
	pfnProxyGenRenderbuffers = glProxyGetProcAddr(handle, "glGenRenderbuffers");
	pfnProxyGenTextures = glProxyGetProcAddr(handle, "glGenTextures");
	pfnProxyGetActiveAttrib = glProxyGetProcAddr(handle, "glGetActiveAttrib");
	pfnProxyGetActiveUniform = glProxyGetProcAddr(handle, "glGetActiveUniform");
	pfnProxyGetAttachedShaders = glProxyGetProcAddr(handle, "glGetAttachedShaders");
	pfnProxyGetAttribLocation = glProxyGetProcAddr(handle, "glGetAttribLocation");
	pfnProxyGetBooleanv = glProxyGetProcAddr(handle, "glGetBooleanv");
	pfnProxyGetBufferParameteriv = glProxyGetProcAddr(handle, "glGetBufferParameteriv");
	pfnProxyGetError = glProxyGetProcAddr(handle, "glGetError");
	pfnProxyGetFloatv = glProxyGetProcAddr(handle, "glGetFloatv");
	pfnProxyGetFramebufferAttachmentParameteriv = glProxyGetProcAddr(handle, "glGetFramebufferAttachmentParameteriv");
	pfnProxyGetIntegerv = glProxyGetProcAddr(handle, "glGetIntegerv");
	pfnProxyGetProgramiv = glProxyGetProcAddr(handle, "glGetProgramiv");
	pfnProxyGetProgramInfoLog = glProxyGetProcAddr(handle, "glGetProgramInfoLog");
	pfnProxyGetRenderbufferParameteriv = glProxyGetProcAddr(handle, "glGetRenderbufferParameteriv");
	pfnProxyGetShaderiv = glProxyGetProcAddr(handle, "glGetShaderiv");
	pfnProxyGetShaderInfoLog = glProxyGetProcAddr(handle, "glGetShaderInfoLog");
	pfnProxyGetShaderPrecisionFormat = glProxyGetProcAddr(handle, "glGetShaderPrecisionFormat");
	pfnProxyGetShaderSource = glProxyGetProcAddr(handle, "glGetShaderSource");
	pfnProxyGetString = glProxyGetProcAddr(handle, "glGetString");
	pfnProxyGetTexParameterfv = glProxyGetProcAddr(handle, "glGetTexParameterfv");
	pfnProxyGetTexParameteriv = glProxyGetProcAddr(handle, "glGetTexParameteriv");
	pfnProxyGetUniformfv = glProxyGetProcAddr(handle, "glGetUniformfv");
	pfnProxyGetUniformiv = glProxyGetProcAddr(handle, "glGetUniformiv");
	pfnProxyGetUniformLocation = glProxyGetProcAddr(handle, "glGetUniformLocation");
	pfnProxyGetVertexAttribfv = glProxyGetProcAddr(handle, "glGetVertexAttribfv");
	pfnProxyGetVertexAttribiv = glProxyGetProcAddr(handle, "glGetVertexAttribiv");
	pfnProxyGetVertexAttribPointerv = glProxyGetProcAddr(handle, "glGetVertexAttribPointerv");
	pfnProxyHint = glProxyGetProcAddr(handle, "glHint");
	pfnProxyIsBuffer = glProxyGetProcAddr(handle, "glIsBuffer");
	pfnProxyIsEnabled = glProxyGetProcAddr(handle, "glIsEnabled");
	pfnProxyIsFramebuffer = glProxyGetProcAddr(handle, "glIsFramebuffer");
	pfnProxyIsProgram = glProxyGetProcAddr(handle, "glIsProgram");
	pfnProxyIsRenderbuffer = glProxyGetProcAddr(handle, "glIsRenderbuffer");
	pfnProxyIsShader = glProxyGetProcAddr(handle, "glIsShader");
	pfnProxyIsTexture = glProxyGetProcAddr(handle, "glIsTexture");
	pfnProxyLineWidth = glProxyGetProcAddr(handle, "glLineWidth");
	pfnProxyLinkProgram = glProxyGetProcAddr(handle, "glLinkProgram");
	pfnProxyPixelStorei = glProxyGetProcAddr(handle, "glPixelStorei");
	pfnProxyPolygonOffset = glProxyGetProcAddr(handle, "glPolygonOffset");
	pfnProxyReadPixels = glProxyGetProcAddr(handle, "glReadPixels");
	pfnProxyReleaseShaderCompiler = glProxyGetProcAddr(handle, "glReleaseShaderCompiler");
	pfnProxyRenderbufferStorage = glProxyGetProcAddr(handle, "glRenderbufferStorage");
	pfnProxySampleCoverage = glProxyGetProcAddr(handle, "glSampleCoverage");
	pfnProxyScissor = glProxyGetProcAddr(handle, "glScissor");
	pfnProxyShaderBinary = glProxyGetProcAddr(handle, "glShaderBinary");
	pfnProxyShaderSource = glProxyGetProcAddr(handle, "glShaderSource");
	pfnProxyStencilFunc = glProxyGetProcAddr(handle, "glStencilFunc");
	pfnProxyStencilFuncSeparate = glProxyGetProcAddr(handle, "glStencilFuncSeparate");
	pfnProxyStencilMask = glProxyGetProcAddr(handle, "glStencilMask");
	pfnProxyStencilMaskSeparate = glProxyGetProcAddr(handle, "glStencilMaskSeparate");
	pfnProxyStencilOp = glProxyGetProcAddr(handle, "glStencilOp");
	pfnProxyStencilOpSeparate = glProxyGetProcAddr(handle, "glStencilOpSeparate");
	pfnProxyTexImage2D = glProxyGetProcAddr(handle, "glTexImage2D");
	pfnProxyTexParameterf = glProxyGetProcAddr(handle, "glTexParameterf");
	pfnProxyTexParameterfv = glProxyGetProcAddr(handle, "glTexParameterfv");
	pfnProxyTexParameteri = glProxyGetProcAddr(handle, "glTexParameteri");
	pfnProxyTexParameteriv = glProxyGetProcAddr(handle, "glTexParameteriv");
	pfnProxyTexSubImage2D = glProxyGetProcAddr(handle, "glTexSubImage2D");
	pfnProxyUniform1f = glProxyGetProcAddr(handle, "glUniform1f");
	pfnProxyUniform1fv = glProxyGetProcAddr(handle, "glUniform1fv");
	pfnProxyUniform1i = glProxyGetProcAddr(handle, "glUniform1i");
	pfnProxyUniform1iv = glProxyGetProcAddr(handle, "glUniform1iv");
	pfnProxyUniform2f = glProxyGetProcAddr(handle, "glUniform2f");
	pfnProxyUniform2fv = glProxyGetProcAddr(handle, "glUniform2fv");
	pfnProxyUniform2i = glProxyGetProcAddr(handle, "glUniform2i");
	pfnProxyUniform2iv = glProxyGetProcAddr(handle, "glUniform2iv");
	pfnProxyUniform3f = glProxyGetProcAddr(handle, "glUniform3f");
	pfnProxyUniform3fv = glProxyGetProcAddr(handle, "glUniform3fv");
	pfnProxyUniform3i = glProxyGetProcAddr(handle, "glUniform3i");
	pfnProxyUniform3iv = glProxyGetProcAddr(handle, "glUniform3iv");
	pfnProxyUniform4f = glProxyGetProcAddr(handle, "glUniform4f");
	pfnProxyUniform4fv = glProxyGetProcAddr(handle, "glUniform4fv");
	pfnProxyUniform4i = glProxyGetProcAddr(handle, "glUniform4i");
	pfnProxyUniform4iv = glProxyGetProcAddr(handle, "glUniform4iv");
	pfnProxyUniformMatrix2fv = glProxyGetProcAddr(handle, "glUniformMatrix2fv");
	pfnProxyUniformMatrix3fv = glProxyGetProcAddr(handle, "glUniformMatrix3fv");
	pfnProxyUniformMatrix4fv = glProxyGetProcAddr(handle, "glUniformMatrix4fv");
	pfnProxyUseProgram = glProxyGetProcAddr(handle, "glUseProgram");
	pfnProxyValidateProgram = glProxyGetProcAddr(handle, "glValidateProgram");
	pfnProxyVertexAttrib1f = glProxyGetProcAddr(handle, "glVertexAttrib1f");
	pfnProxyVertexAttrib1fv = glProxyGetProcAddr(handle, "glVertexAttrib1fv");
	pfnProxyVertexAttrib2f = glProxyGetProcAddr(handle, "glVertexAttrib2f");
	pfnProxyVertexAttrib2fv = glProxyGetProcAddr(handle, "glVertexAttrib2fv");
	pfnProxyVertexAttrib3f = glProxyGetProcAddr(handle, "glVertexAttrib3f");
	pfnProxyVertexAttrib3fv = glProxyGetProcAddr(handle, "glVertexAttrib3fv");
	pfnProxyVertexAttrib4f = glProxyGetProcAddr(handle, "glVertexAttrib4f");
	pfnProxyVertexAttrib4fv = glProxyGetProcAddr(handle, "glVertexAttrib4fv");
	pfnProxyVertexAttribPointer = glProxyGetProcAddr(handle, "glVertexAttribPointer");
	pfnProxyViewport = glProxyGetProcAddr(handle, "glViewport");
	
	return 0;
}

extern int glProxyExtInitGLES2(void *handle);
extern int glProxyExtInitGL(void *handle);
extern int glxProxyInit(void *handle);
extern int glxProxyExtInit(void *handle);

int glProxyInit(PROXY_BACKEND_OPTION *pProxyBackend)
{
	/* Select backend first */
	glProxyBackendSelection(pProxyBackend, &global_proxy_context);

	if (UNKNOWN_BACKEND == global_proxy_context.current_backend) {
		printf("Error: No OpenGL or OpenGL ES2.0 renderer available\n");
		return -1;
	}

	printf("%s backend is selected for rendering\n", (global_proxy_context.current_backend == OPENGL_BACKEND) ? "OpenGL" : "OpenGL ES2.0");

	/* Load selected backend, and resolve glXGetProcAddress/eglGetProcAddress */
	glProxyEarlyResolve(&global_proxy_context);

	/* Resolve symbols */
	if (OPENGL_BACKEND == global_proxy_context.current_backend) {
		glProxyCoreInit(global_proxy_context.gl_handle);
		glProxyExtInitGL(global_proxy_context.gl_handle);
		glxProxyInit(global_proxy_context.gl_handle);
		glxProxyExtInit(global_proxy_context.gl_handle);
	}
	else if (OPENGL_ES20_BACKEND == global_proxy_context.current_backend) {
		glProxyCoreInit(global_proxy_context.gles2_handle);
		glProxyExtInitGLES2(global_proxy_context.gles2_handle);
	}
	else {
		/* should not get here */
		return -1;
	}

	return 0;
}

int glProxyClose(void)
{
	if (OPENGL_ES20_BACKEND == global_proxy_context.current_backend)
		DestroyEGLRenderingContext(&global_proxy_context);

	if (global_proxy_context.x_dpy) {
		XCloseDisplay(global_proxy_context.x_dpy);
		global_proxy_context.x_dpy = NULL;
	}

	if (global_proxy_context.gl_handle) {
		glProxyCloseLibrary(global_proxy_context.gl_handle);
		global_proxy_context.gl_handle = NULL;
	}

	if (global_proxy_context.gles2_handle) {
		glProxyCloseLibrary(global_proxy_context.gles2_handle);
		global_proxy_context.gles2_handle = NULL;
        }

	if (global_proxy_context.egl_handle) {
		glProxyCloseLibrary(global_proxy_context.egl_handle);
		global_proxy_context.egl_handle = NULL;
	}

	global_proxy_context.current_backend = UNKNOWN_BACKEND;

	return 0;
}
